# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from enum import IntEnum

from xdrlib3 import Packer, Unpacker

__all__ = ["AllowTrustResultCode"]


class AllowTrustResultCode(IntEnum):
    """
    XDR Source Code::

        enum AllowTrustResultCode
        {
            // codes considered as "success" for the operation
            ALLOW_TRUST_SUCCESS = 0,
            // codes considered as "failure" for the operation
            ALLOW_TRUST_MALFORMED = -1,     // asset is not ASSET_TYPE_ALPHANUM
            ALLOW_TRUST_NO_TRUST_LINE = -2, // trustor does not have a trustline
                                            // source account does not require trust
            ALLOW_TRUST_TRUST_NOT_REQUIRED = -3,
            ALLOW_TRUST_CANT_REVOKE = -4,      // source account can't revoke trust,
            ALLOW_TRUST_SELF_NOT_ALLOWED = -5, // trusting self is not allowed
            ALLOW_TRUST_LOW_RESERVE = -6       // claimable balances can't be created
                                               // on revoke due to low reserves
        };
    """

    ALLOW_TRUST_SUCCESS = 0
    ALLOW_TRUST_MALFORMED = -1
    ALLOW_TRUST_NO_TRUST_LINE = -2
    ALLOW_TRUST_TRUST_NOT_REQUIRED = -3
    ALLOW_TRUST_CANT_REVOKE = -4
    ALLOW_TRUST_SELF_NOT_ALLOWED = -5
    ALLOW_TRUST_LOW_RESERVE = -6

    def pack(self, packer: Packer) -> None:
        packer.pack_int(self.value)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> AllowTrustResultCode:
        value = unpacker.unpack_int()
        return cls(value)

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> AllowTrustResultCode:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> AllowTrustResultCode:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .base import Boolean
from .contract_event import ContractEvent

__all__ = ["DiagnosticEvent"]


class DiagnosticEvent:
    """
    XDR Source Code::

        struct DiagnosticEvent
        {
            bool inSuccessfulContractCall;
            ContractEvent event;
        };
    """

    def __init__(
        self,
        in_successful_contract_call: bool,
        event: ContractEvent,
    ) -> None:
        self.in_successful_contract_call = in_successful_contract_call
        self.event = event

    def pack(self, packer: Packer) -> None:
        Boolean(self.in_successful_contract_call).pack(packer)
        self.event.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> DiagnosticEvent:
        in_successful_contract_call = Boolean.unpack(unpacker)
        event = ContractEvent.unpack(unpacker)
        return cls(
            in_successful_contract_call=in_successful_contract_call,
            event=event,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> DiagnosticEvent:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> DiagnosticEvent:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.in_successful_contract_call,
                self.event,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.in_successful_contract_call == other.in_successful_contract_call
            and self.event == other.event
        )

    def __repr__(self):
        out = [
            f"in_successful_contract_call={self.in_successful_contract_call}",
            f"event={self.event}",
        ]
        return f"<DiagnosticEvent [{', '.join(out)}]>"



import asyncio
import dataclasses
import time
from typing import Callable, Generic, Optional, Set, TypeVar, Union

from .. import Address, Keypair, SorobanDataBuilder, SorobanServerAsync, xdr
from ..auth import authorize_entry
from ..base_soroban_server import _assemble_transaction
from ..operation import InvokeHostFunction
from ..soroban_rpc import (
    GetTransactionResponse,
    GetTransactionStatus,
    RestorePreamble,
    SendTransactionResponse,
    SendTransactionStatus,
    SimulateHostFunctionResult,
    SimulateTransactionResponse,
)
from ..transaction_builder import TransactionBuilder
from ..transaction_envelope import TransactionEnvelope
from ..xdr import TransactionMeta
from .exceptions import *

T = TypeVar("T")

__all__ = ["AssembledTransactionAsync"]


class AssembledTransactionAsync(Generic[T]):
    """A class representing an assembled Soroban transaction that can be simulated and sent.

    The lifecycle of a transaction typically follows these steps:
        1. Construct the transaction (usually via a Client)
        2. Simulate the transaction
        3. Sign the transaction
        4. Submit the transaction

    :param transaction_builder: The transaction builder including the operation to invoke
    :param server: The Soroban server instance to use
    :param transaction_signer: Optional keypair for signing transactions, if you don't need to submit the transaction, you can set this to `None`.
    :param parse_result_xdr_fn: Optional function to parse XDR results, keep `None` for raw XDR
    :param submit_timeout: Timeout in seconds for transaction submission (default: 180s)
    """

    def __init__(
        self,
        transaction_builder: TransactionBuilder,
        server: SorobanServerAsync,
        transaction_signer: Keypair = None,
        parse_result_xdr_fn: Optional[Callable[[xdr.SCVal], T]] = None,
        submit_timeout: int = 180,
    ):
        self.server = server
        self.submit_timeout = submit_timeout

        self.transaction_signer = transaction_signer
        self.parse_result_xdr_fn = parse_result_xdr_fn

        self.transaction_builder: TransactionBuilder = transaction_builder
        self.built_transaction: Optional[TransactionEnvelope] = None

        self.simulation: Optional[SimulateTransactionResponse] = None
        self._simulation_result: Optional[SimulateHostFunctionResult] = None
        self._simulation_transaction_data: Optional[xdr.SorobanTransactionData] = None

        self.send_transaction_response: Optional[SendTransactionResponse] = None
        self.get_transaction_response: Optional[GetTransactionResponse] = None

    async def simulate(self, restore: bool = True) -> "AssembledTransactionAsync":
        """Simulates the transaction on the network.

        Must be called before signing or submitting the transaction.
        Will automatically restore required contract state if restore to True.

        :param restore: Whether to automatically restore contract state if needed, defaults to True
        :return: Self for chaining
        :raises: :exc:`SimulationFailedError <stellar_sdk.contract.exceptions.SimulationFailedError>`: If the simulation fails
        :raises: :exc:`ExpiredStateError <stellar_sdk.contract.exceptions.ExpiredStateError>`: If state restoration failed
        """
        self._simulation_result = None
        self._simulation_transaction_data = None

        source = await self.server.load_account(
            self.transaction_builder.source_account.account.account_id
        )
        self.transaction_builder.source_account.sequence = source.sequence

        built_tx = self.transaction_builder.build()
        self.simulation = await self.server.simulate_transaction(built_tx)

        if restore and self.simulation.restore_preamble:
            try:
                await self.restore_footprint(self.simulation.restore_preamble)
            except (
                SimulationFailedError,
                TransactionStillPendingError,
                SendTransactionFailedError,
                TransactionFailedError,
            ) as e:
                raise RestorationFailureError(
                    "Failed to restore contract data.", self
                ) from e
            await self.simulate()

        if self.simulation.error is not None:
            raise SimulationFailedError(
                f"Transaction simulation failed: {self.simulation.error}", self
            )
        self.built_transaction = _assemble_transaction(built_tx, self.simulation)
        return self

    async def sign_and_submit(
        self, transaction_signer: Keypair = None, force: bool = False
    ) -> Union[T, xdr.SCVal]:
        """Signs and submits the transaction in one step.

        A convenience method combining sign() and submit().

        :param transaction_signer: transaction_signer: Optional keypair to sign with (overrides instance signer)
        :param force: Whether to sign and submit even if the transaction is a read call
        :return: The value returned by the invoked function, parsed if parse_result_xdr_fn was set, otherwise raw XDR
        """
        self.sign(force=force, transaction_signer=transaction_signer)
        return await self.submit()

    def sign(
        self, transaction_signer: Keypair = None, force: bool = False
    ) -> "AssembledTransactionAsync":
        """Signs the transaction.

        :param transaction_signer: Optional keypair to sign with (overrides instance signer)
        :param force: Whether to sign even if the transaction is a read call
        :return: Self for chaining
        :raises: :exc:`NotYetSimulatedError <stellar_sdk.contract.exceptions.NotYetSimulatedError>`: If the transaction has not been simulated
        :raises: :exc:`NoSignatureNeededError <stellar_sdk.contract.exceptions.NoSignatureNeededError>`: If the transaction is a read call
        :raises: :exc:`NeedsMoreSignaturesError <stellar_sdk.contract.exceptions.NeedsMoreSignaturesError>`: If the transaction requires more signatures for authorization entries.
        """
        if not self.built_transaction:
            raise NotYetSimulatedError("Transaction has not yet been simulated.", self)

        if not force and self.is_read_call():
            raise NoSignatureNeededError(
                "This is a read call. It requires no signature or submitting. Set force=True to sign and submit anyway.",
                self,
            )

        transaction_signer = transaction_signer or self.transaction_signer
        if not transaction_signer:
            raise ValueError(
                "You must provide a sign_transaction_func to sign the transaction, either here or in the constructor."
            )

        sigs_needed = [
            s for s in self.needs_non_invoker_signing_by() if not s.startswith("C")
        ]
        if sigs_needed:
            raise NeedsMoreSignaturesError(
                f"`Transaction requires signatures from {sigs_needed}`. See `needs_non_invoker_signing_by` for details.",
                self,
            )

        self.built_transaction.sign(transaction_signer)
        return self

    async def sign_auth_entries(
        self, auth_entries_signer: Keypair, valid_until_ledger_sequence: int = None
    ) -> "AssembledTransactionAsync":
        """Signs the transaction's authorization entries.

        :param auth_entries_signer: The keypair to sign the authorization entries.
        :param valid_until_ledger_sequence: Optional ledger sequence until which the authorization is valid, if not set, defaults to 100 ledgers from the current ledger.
        :return: Self for chaining
        :raises: :exc:`NotYetSimulatedError <stellar_sdk.contract.exceptions.NotYetSimulatedError>`: If the transaction has not been simulated
        """
        if valid_until_ledger_sequence is None:
            valid_until_ledger_sequence = (
                await self.server.get_latest_ledger()
            ).sequence + 100

        if not self.built_transaction:
            raise NotYetSimulatedError("Transaction has not yet been simulated.", self)

        op = self.built_transaction.transaction.operations[0]
        assert isinstance(op, InvokeHostFunction)
        for i, e in enumerate(op.auth):
            if (
                e.credentials.type
                == xdr.SorobanCredentialsType.SOROBAN_CREDENTIALS_SOURCE_ACCOUNT
            ):
                continue
            assert e.credentials.address is not None
            if (
                Address.from_xdr_sc_address(e.credentials.address.address).address
                != auth_entries_signer.public_key
            ):
                continue
            op.auth[i] = authorize_entry(
                e,
                auth_entries_signer,
                valid_until_ledger_sequence,
                self.built_transaction.network_passphrase,
            )
        return self

    async def submit(self) -> Union[T, xdr.SCVal]:
        """Submits the transaction to the network.

        It will send the transaction to the network and wait for the result.

        :return: The value returned by the invoked function, parsed if parse_result_xdr_fn was set, otherwise raw XDR
        :raises: :exc:`SendTransactionFailedError <stellar_sdk.contract.exceptions.SendTransactionFailedError>`: If sending the transaction fails
        :raises: :exc:`TransactionStillPendingError <stellar_sdk.contract.exceptions.TransactionStillPendingError>`: If the transaction is still pending after the timeout, you can re-call this method to wait longer
        :raises: :exc:`TransactionFailedError <stellar_sdk.contract.exceptions.TransactionFailedError>`: If the transaction fails
        """
        response = await self._submit()
        assert response.result_meta_xdr is not None
        transaction_meta = TransactionMeta.from_xdr(response.result_meta_xdr)
        assert transaction_meta.v3 is not None
        assert transaction_meta.v3.soroban_meta is not None
        result_val = transaction_meta.v3.soroban_meta.return_value
        return (
            self.parse_result_xdr_fn(result_val)
            if self.parse_result_xdr_fn
            else result_val
        )

    async def _submit(self) -> GetTransactionResponse:
        if not self.built_transaction:
            raise NotYetSimulatedError("Transaction has not yet been simulated.", self)

        if not self.send_transaction_response:
            self.send_transaction_response = await self.server.send_transaction(
                self.built_transaction
            )
            if self.send_transaction_response.status != SendTransactionStatus.PENDING:
                raise SendTransactionFailedError(
                    f"Sending the transaction to the network failed!\n{self.send_transaction_response.model_dump_json()}",
                    self,
                )

        tx_hash = self.send_transaction_response.hash
        self.get_transaction_response = (
            await _with_exponential_backoff(
                lambda: self.server.get_transaction(tx_hash),
                lambda resp: resp.status == GetTransactionStatus.NOT_FOUND,
                self.submit_timeout,
            )
        )[-1]

        assert self.get_transaction_response is not None
        if self.get_transaction_response.status == GetTransactionStatus.SUCCESS:
            return self.get_transaction_response
        if self.get_transaction_response.status == GetTransactionStatus.NOT_FOUND:
            raise TransactionStillPendingError(
                f"Waited {self.submit_timeout} seconds for transaction to complete, but it did not. "
                f"Returning anyway. You can call result() to await the result later "
                f"or check the status of the transaction manually.",
                self,
            )
        elif self.get_transaction_response.status == GetTransactionStatus.FAILED:
            raise TransactionFailedError(f"Transaction failed.", self)
        else:
            raise ValueError("Unexpected transaction status.")

    def needs_non_invoker_signing_by(
        self, include_already_signed: bool = False
    ) -> Set[str]:
        """Get the addresses that need to sign the authorization entries.

        :param include_already_signed: Whether to include addresses that have already signed the authorization entries.
        :return: The addresses that need to sign the authorization entries.
        :raises: :exc:`NotYetSimulatedError <stellar_sdk.contract.exceptions.NotYetSimulatedError>`: If the transaction has not been simulated
        """
        if not self.built_transaction:
            raise NotYetSimulatedError("Transaction has not yet been simulated.", self)

        invoke_host_function_op = self.built_transaction.transaction.operations[0]
        if not isinstance(invoke_host_function_op, InvokeHostFunction):
            raise ValueError("Unexpected operation type.")

        result = set()
        for entry in invoke_host_function_op.auth or []:
            if (
                entry.credentials.type
                == xdr.SorobanCredentialsType.SOROBAN_CREDENTIALS_ADDRESS
            ):
                assert entry.credentials.address is not None

                if (
                    include_already_signed
                    or entry.credentials.address.signature.type
                    == xdr.SCValType.SCV_VOID
                ):
                    address = Address.from_xdr_sc_address(
                        entry.credentials.address.address
                    ).address
                    result.add(address)
        return result

    def result(self) -> Union[T, xdr.SCVal]:
        """Get the result of the function invocation from the simulation.

        :return: The value returned by the invoked function, parsed if parse_result_xdr_fn was set, otherwise raw XDR
        """
        raw_result = xdr.SCVal.from_xdr(self._simulation_data().result.xdr)
        if self.parse_result_xdr_fn:
            return self.parse_result_xdr_fn(raw_result)
        return raw_result

    def is_read_call(self) -> bool:
        """Check if the transaction is a read call.

        :return: True if the transaction is a read call, False otherwise
        :raises: :exc:`NotYetSimulatedError <stellar_sdk.contract.exceptions.NotYetSimulatedError>`: If the transaction has not been simulated
        """
        simulation_data = self._simulation_data
        auths = simulation_data().result.auth
        writes = simulation_data().transaction_data.resources.footprint.read_write
        return not auths and not writes

    def to_xdr(self):
        """Get the XDR representation of the transaction envelope.

        :return: The XDR representation of the transaction envelope
        """
        return self.built_transaction.to_xdr()

    async def restore_footprint(self, restore_preamble: RestorePreamble) -> None:
        if not self.transaction_signer:
            raise ValueError(
                "For automatic restore to work you must provide a transaction_signer when initializing AssembledTransactionAsync."
            )
        restore_tx = (
            TransactionBuilder(
                self.transaction_builder.source_account,
                self.transaction_builder.network_passphrase,
                self.transaction_builder.base_fee,
            )
            .append_restore_footprint_op()
            .set_soroban_data(
                SorobanDataBuilder.from_xdr(restore_preamble.transaction_data)
            )
            .time_bounds(0, 0)
        )
        restore_assembled: AssembledTransactionAsync = AssembledTransactionAsync(
            restore_tx,
            self.server,
            self.transaction_signer,
            None,
            submit_timeout=self.submit_timeout,
        )
        await restore_assembled.simulate(restore=False)
        restore_assembled.sign()
        await restore_assembled._submit()

    def _simulation_data(
        self,
    ) -> "SimulationData":
        if self._simulation_result and self._simulation_transaction_data:
            return SimulationData(
                self._simulation_result, self._simulation_transaction_data
            )

        if not self.simulation:
            raise NotYetSimulatedError("Transaction has not yet been simulated.", self)

        if self.simulation.restore_preamble:
            raise ExpiredStateError(
                "You need to restore some contract state before you can invoke this method. "
                + "You can set `restore` to true in order to "
                + "automatically restore the contract state when needed.",
                self,
            )

        # SimulateHostFunctionResult(auth=[], xdr='AAAAAQ==') for no return function (void)
        assert self.simulation.results is not None
        assert self.simulation.transaction_data is not None
        self._simulation_result = self.simulation.results[0]
        self._simulation_transaction_data = xdr.SorobanTransactionData.from_xdr(
            self.simulation.transaction_data
        )
        return SimulationData(
            self._simulation_result, self._simulation_transaction_data
        )


@dataclasses.dataclass
class SimulationData:
    result: SimulateHostFunctionResult
    transaction_data: xdr.SorobanTransactionData


async def _with_exponential_backoff(
    fn,
    keep_waiting_if,
    timeout: float,
    exponential_factor: float = 1.5,
    verbose: bool = False,
):
    """Keep calling a function for timeout seconds if keep_waiting_if is true.
    Returns an array of all attempts to call the function.

    :param fn: Function to call repeatedly
    :param keep_waiting_if: Condition to check when deciding whether to call fn again
    :param timeout: How long to wait between first and second call
    :param exponential_factor: What to multiply timeout by each subsequent attempt
    :param verbose: Whether to log extra info, for debug only
    """
    attempts = []

    count = 0
    attempts.append(await fn())
    if not keep_waiting_if(attempts[-1]):
        return attempts

    wait_until = time.time() + timeout
    wait_time = 1.0  # seconds
    total_wait_time = wait_time

    while time.time() < wait_until and keep_waiting_if(attempts[-1]):
        count += 1

        # Log waiting time if verbose
        if verbose:
            print(
                f"Waiting {wait_time * 1000} ms before trying again "
                f"(bringing the total wait time to {total_wait_time * 1000} ms "
                f"so far, of total {timeout * 1000} ms)"
            )

        await asyncio.sleep(wait_time)

        wait_time *= exponential_factor
        if wait_time >= 10:
            wait_time = 10

        if time.time() + wait_time > wait_until:
            wait_time = wait_until - time.time()
            if verbose:
                print(f"was gonna wait too long; new waitTime: {wait_time * 1000} ms")

        total_wait_time += wait_time
        attempts.append(await fn())

        if verbose and keep_waiting_if(attempts[-1]):
            print(
                f"{count}. Called fn; "
                f"{len(attempts)} prev attempts. "
                f"Most recent: {attempts[-1].model_dump_json()}"
            )

    return attempts



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .stellar_value import StellarValue
from .stored_transaction_set import StoredTransactionSet
from .uint32 import Uint32

__all__ = ["StoredDebugTransactionSet"]


class StoredDebugTransactionSet:
    """
    XDR Source Code::

                                                                struct StoredDebugTransactionSet
                                                                {
                                                                        StoredTransactionSet txSet;
                                                                        uint32 ledgerSeq;
                                                                        StellarValue scpValue;
                                                                };
    """

    def __init__(
        self,
        tx_set: StoredTransactionSet,
        ledger_seq: Uint32,
        scp_value: StellarValue,
    ) -> None:
        self.tx_set = tx_set
        self.ledger_seq = ledger_seq
        self.scp_value = scp_value

    def pack(self, packer: Packer) -> None:
        self.tx_set.pack(packer)
        self.ledger_seq.pack(packer)
        self.scp_value.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> StoredDebugTransactionSet:
        tx_set = StoredTransactionSet.unpack(unpacker)
        ledger_seq = Uint32.unpack(unpacker)
        scp_value = StellarValue.unpack(unpacker)
        return cls(
            tx_set=tx_set,
            ledger_seq=ledger_seq,
            scp_value=scp_value,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> StoredDebugTransactionSet:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> StoredDebugTransactionSet:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.tx_set,
                self.ledger_seq,
                self.scp_value,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.tx_set == other.tx_set
            and self.ledger_seq == other.ledger_seq
            and self.scp_value == other.scp_value
        )

    def __repr__(self):
        out = [
            f"tx_set={self.tx_set}",
            f"ledger_seq={self.ledger_seq}",
            f"scp_value={self.scp_value}",
        ]
        return f"<StoredDebugTransactionSet [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from typing import List

from xdrlib3 import Packer, Unpacker

from .base import Integer
from .operation_meta import OperationMeta
from .transaction_meta_v1 import TransactionMetaV1
from .transaction_meta_v2 import TransactionMetaV2
from .transaction_meta_v3 import TransactionMetaV3

__all__ = ["TransactionMeta"]


class TransactionMeta:
    """
    XDR Source Code::

        union TransactionMeta switch (int v)
        {
        case 0:
            OperationMeta operations<>;
        case 1:
            TransactionMetaV1 v1;
        case 2:
            TransactionMetaV2 v2;
        case 3:
            TransactionMetaV3 v3;
        };
    """

    def __init__(
        self,
        v: int,
        operations: List[OperationMeta] = None,
        v1: TransactionMetaV1 = None,
        v2: TransactionMetaV2 = None,
        v3: TransactionMetaV3 = None,
    ) -> None:
        _expect_max_length = 4294967295
        if operations and len(operations) > _expect_max_length:
            raise ValueError(
                f"The maximum length of `operations` should be {_expect_max_length}, but got {len(operations)}."
            )
        self.v = v
        self.operations = operations
        self.v1 = v1
        self.v2 = v2
        self.v3 = v3

    def pack(self, packer: Packer) -> None:
        Integer(self.v).pack(packer)
        if self.v == 0:
            if self.operations is None:
                raise ValueError("operations should not be None.")
            packer.pack_uint(len(self.operations))
            for operations_item in self.operations:
                operations_item.pack(packer)
            return
        if self.v == 1:
            if self.v1 is None:
                raise ValueError("v1 should not be None.")
            self.v1.pack(packer)
            return
        if self.v == 2:
            if self.v2 is None:
                raise ValueError("v2 should not be None.")
            self.v2.pack(packer)
            return
        if self.v == 3:
            if self.v3 is None:
                raise ValueError("v3 should not be None.")
            self.v3.pack(packer)
            return

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> TransactionMeta:
        v = Integer.unpack(unpacker)
        if v == 0:
            length = unpacker.unpack_uint()
            operations = []
            for _ in range(length):
                operations.append(OperationMeta.unpack(unpacker))
            return cls(v=v, operations=operations)
        if v == 1:
            v1 = TransactionMetaV1.unpack(unpacker)
            return cls(v=v, v1=v1)
        if v == 2:
            v2 = TransactionMetaV2.unpack(unpacker)
            return cls(v=v, v2=v2)
        if v == 3:
            v3 = TransactionMetaV3.unpack(unpacker)
            return cls(v=v, v3=v3)
        return cls(v=v)

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> TransactionMeta:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> TransactionMeta:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.v,
                self.operations,
                self.v1,
                self.v2,
                self.v3,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.v == other.v
            and self.operations == other.operations
            and self.v1 == other.v1
            and self.v2 == other.v2
            and self.v3 == other.v3
        )

    def __repr__(self):
        out = []
        out.append(f"v={self.v}")
        (
            out.append(f"operations={self.operations}")
            if self.operations is not None
            else None
        )
        out.append(f"v1={self.v1}") if self.v1 is not None else None
        out.append(f"v2={self.v2}") if self.v2 is not None else None
        out.append(f"v3={self.v3}") if self.v3 is not None else None
        return f"<TransactionMeta [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .ledger_entry import LedgerEntry
from .uint32 import Uint32

__all__ = ["ColdArchiveArchivedLeaf"]


class ColdArchiveArchivedLeaf:
    """
    XDR Source Code::

        struct ColdArchiveArchivedLeaf
        {
            uint32 index;
            LedgerEntry archivedEntry;
        };
    """

    def __init__(
        self,
        index: Uint32,
        archived_entry: LedgerEntry,
    ) -> None:
        self.index = index
        self.archived_entry = archived_entry

    def pack(self, packer: Packer) -> None:
        self.index.pack(packer)
        self.archived_entry.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> ColdArchiveArchivedLeaf:
        index = Uint32.unpack(unpacker)
        archived_entry = LedgerEntry.unpack(unpacker)
        return cls(
            index=index,
            archived_entry=archived_entry,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> ColdArchiveArchivedLeaf:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> ColdArchiveArchivedLeaf:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.index,
                self.archived_entry,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.index == other.index and self.archived_entry == other.archived_entry

    def __repr__(self):
        out = [
            f"index={self.index}",
            f"archived_entry={self.archived_entry}",
        ]
        return f"<ColdArchiveArchivedLeaf [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
#: const MASK_ACCOUNT_FLAGS = 0x7;
MASK_ACCOUNT_FLAGS: int = 0x7
#: const MASK_ACCOUNT_FLAGS_V17 = 0xF;
MASK_ACCOUNT_FLAGS_V17: int = 0xF
#: const MAX_SIGNERS = 20;
MAX_SIGNERS: int = 20
#: const MASK_TRUSTLINE_FLAGS = 1;
MASK_TRUSTLINE_FLAGS: int = 1
#: const MASK_TRUSTLINE_FLAGS_V13 = 3;
MASK_TRUSTLINE_FLAGS_V13: int = 3
#: const MASK_TRUSTLINE_FLAGS_V17 = 7;
MASK_TRUSTLINE_FLAGS_V17: int = 7
#: const MASK_OFFERENTRY_FLAGS = 1;
MASK_OFFERENTRY_FLAGS: int = 1
#: const MASK_CLAIMABLE_BALANCE_FLAGS = 0x1;
MASK_CLAIMABLE_BALANCE_FLAGS: int = 0x1
#: const MASK_LEDGER_HEADER_FLAGS = 0x7;
MASK_LEDGER_HEADER_FLAGS: int = 0x7
#: const AUTH_MSG_FLAG_FLOW_CONTROL_BYTES_REQUESTED = 200;
AUTH_MSG_FLAG_FLOW_CONTROL_BYTES_REQUESTED: int = 200
#: const TX_ADVERT_VECTOR_MAX_SIZE = 1000;
TX_ADVERT_VECTOR_MAX_SIZE: int = 1000
#: const TX_DEMAND_VECTOR_MAX_SIZE = 1000;
TX_DEMAND_VECTOR_MAX_SIZE: int = 1000
#: const MAX_OPS_PER_TX = 100;
MAX_OPS_PER_TX: int = 100
#: const LIQUIDITY_POOL_FEE_V18 = 30;
LIQUIDITY_POOL_FEE_V18: int = 30
#: const SC_SPEC_DOC_LIMIT = 1024;
SC_SPEC_DOC_LIMIT: int = 1024
#: const SCSYMBOL_LIMIT = 32;
SCSYMBOL_LIMIT: int = 32
#: const CONTRACT_COST_COUNT_LIMIT = 1024;
CONTRACT_COST_COUNT_LIMIT: int = 1024



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .signature import Signature
from .signature_hint import SignatureHint

__all__ = ["DecoratedSignature"]


class DecoratedSignature:
    """
    XDR Source Code::

        struct DecoratedSignature
        {
            SignatureHint hint;  // last 4 bytes of the public key, used as a hint
            Signature signature; // actual signature
        };
    """

    def __init__(
        self,
        hint: SignatureHint,
        signature: Signature,
    ) -> None:
        self.hint = hint
        self.signature = signature

    def pack(self, packer: Packer) -> None:
        self.hint.pack(packer)
        self.signature.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> DecoratedSignature:
        hint = SignatureHint.unpack(unpacker)
        signature = Signature.unpack(unpacker)
        return cls(
            hint=hint,
            signature=signature,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> DecoratedSignature:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> DecoratedSignature:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.hint,
                self.signature,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.hint == other.hint and self.signature == other.signature

    def __repr__(self):
        out = [
            f"hint={self.hint}",
            f"signature={self.signature}",
        ]
        return f"<DecoratedSignature [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .int64 import Int64
from .uint64 import Uint64

__all__ = ["Int256Parts"]


class Int256Parts:
    """
    XDR Source Code::

        struct Int256Parts {
            int64 hi_hi;
            uint64 hi_lo;
            uint64 lo_hi;
            uint64 lo_lo;
        };
    """

    def __init__(
        self,
        hi_hi: Int64,
        hi_lo: Uint64,
        lo_hi: Uint64,
        lo_lo: Uint64,
    ) -> None:
        self.hi_hi = hi_hi
        self.hi_lo = hi_lo
        self.lo_hi = lo_hi
        self.lo_lo = lo_lo

    def pack(self, packer: Packer) -> None:
        self.hi_hi.pack(packer)
        self.hi_lo.pack(packer)
        self.lo_hi.pack(packer)
        self.lo_lo.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> Int256Parts:
        hi_hi = Int64.unpack(unpacker)
        hi_lo = Uint64.unpack(unpacker)
        lo_hi = Uint64.unpack(unpacker)
        lo_lo = Uint64.unpack(unpacker)
        return cls(
            hi_hi=hi_hi,
            hi_lo=hi_lo,
            lo_hi=lo_hi,
            lo_lo=lo_lo,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> Int256Parts:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> Int256Parts:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.hi_hi,
                self.hi_lo,
                self.lo_hi,
                self.lo_lo,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.hi_hi == other.hi_hi
            and self.hi_lo == other.hi_lo
            and self.lo_hi == other.lo_hi
            and self.lo_lo == other.lo_lo
        )

    def __repr__(self):
        out = [
            f"hi_hi={self.hi_hi}",
            f"hi_lo={self.hi_lo}",
            f"lo_hi={self.lo_hi}",
            f"lo_lo={self.lo_lo}",
        ]
        return f"<Int256Parts [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .int32 import Int32
from .trust_line_entry_extension_v2_ext import TrustLineEntryExtensionV2Ext

__all__ = ["TrustLineEntryExtensionV2"]


class TrustLineEntryExtensionV2:
    """
    XDR Source Code::

        struct TrustLineEntryExtensionV2
        {
            int32 liquidityPoolUseCount;

            union switch (int v)
            {
            case 0:
                void;
            }
            ext;
        };
    """

    def __init__(
        self,
        liquidity_pool_use_count: Int32,
        ext: TrustLineEntryExtensionV2Ext,
    ) -> None:
        self.liquidity_pool_use_count = liquidity_pool_use_count
        self.ext = ext

    def pack(self, packer: Packer) -> None:
        self.liquidity_pool_use_count.pack(packer)
        self.ext.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> TrustLineEntryExtensionV2:
        liquidity_pool_use_count = Int32.unpack(unpacker)
        ext = TrustLineEntryExtensionV2Ext.unpack(unpacker)
        return cls(
            liquidity_pool_use_count=liquidity_pool_use_count,
            ext=ext,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> TrustLineEntryExtensionV2:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> TrustLineEntryExtensionV2:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.liquidity_pool_use_count,
                self.ext,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.liquidity_pool_use_count == other.liquidity_pool_use_count
            and self.ext == other.ext
        )

    def __repr__(self):
        out = [
            f"liquidity_pool_use_count={self.liquidity_pool_use_count}",
            f"ext={self.ext}",
        ]
        return f"<TrustLineEntryExtensionV2 [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .hash import Hash
from .scp_ballot import SCPBallot
from .uint32 import Uint32

__all__ = ["SCPStatementConfirm"]


class SCPStatementConfirm:
    """
    XDR Source Code::

        struct
                {
                    SCPBallot ballot;   // b
                    uint32 nPrepared;   // p.n
                    uint32 nCommit;     // c.n
                    uint32 nH;          // h.n
                    Hash quorumSetHash; // D
                }
    """

    def __init__(
        self,
        ballot: SCPBallot,
        n_prepared: Uint32,
        n_commit: Uint32,
        n_h: Uint32,
        quorum_set_hash: Hash,
    ) -> None:
        self.ballot = ballot
        self.n_prepared = n_prepared
        self.n_commit = n_commit
        self.n_h = n_h
        self.quorum_set_hash = quorum_set_hash

    def pack(self, packer: Packer) -> None:
        self.ballot.pack(packer)
        self.n_prepared.pack(packer)
        self.n_commit.pack(packer)
        self.n_h.pack(packer)
        self.quorum_set_hash.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> SCPStatementConfirm:
        ballot = SCPBallot.unpack(unpacker)
        n_prepared = Uint32.unpack(unpacker)
        n_commit = Uint32.unpack(unpacker)
        n_h = Uint32.unpack(unpacker)
        quorum_set_hash = Hash.unpack(unpacker)
        return cls(
            ballot=ballot,
            n_prepared=n_prepared,
            n_commit=n_commit,
            n_h=n_h,
            quorum_set_hash=quorum_set_hash,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> SCPStatementConfirm:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> SCPStatementConfirm:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.ballot,
                self.n_prepared,
                self.n_commit,
                self.n_h,
                self.quorum_set_hash,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.ballot == other.ballot
            and self.n_prepared == other.n_prepared
            and self.n_commit == other.n_commit
            and self.n_h == other.n_h
            and self.quorum_set_hash == other.quorum_set_hash
        )

    def __repr__(self):
        out = [
            f"ballot={self.ballot}",
            f"n_prepared={self.n_prepared}",
            f"n_commit={self.n_commit}",
            f"n_h={self.n_h}",
            f"quorum_set_hash={self.quorum_set_hash}",
        ]
        return f"<SCPStatementConfirm [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .asset import Asset
from .contract_id_preimage_from_address import ContractIDPreimageFromAddress
from .contract_id_preimage_type import ContractIDPreimageType

__all__ = ["ContractIDPreimage"]


class ContractIDPreimage:
    """
    XDR Source Code::

        union ContractIDPreimage switch (ContractIDPreimageType type)
        {
        case CONTRACT_ID_PREIMAGE_FROM_ADDRESS:
            struct
            {
                SCAddress address;
                uint256 salt;
            } fromAddress;
        case CONTRACT_ID_PREIMAGE_FROM_ASSET:
            Asset fromAsset;
        };
    """

    def __init__(
        self,
        type: ContractIDPreimageType,
        from_address: ContractIDPreimageFromAddress = None,
        from_asset: Asset = None,
    ) -> None:
        self.type = type
        self.from_address = from_address
        self.from_asset = from_asset

    def pack(self, packer: Packer) -> None:
        self.type.pack(packer)
        if self.type == ContractIDPreimageType.CONTRACT_ID_PREIMAGE_FROM_ADDRESS:
            if self.from_address is None:
                raise ValueError("from_address should not be None.")
            self.from_address.pack(packer)
            return
        if self.type == ContractIDPreimageType.CONTRACT_ID_PREIMAGE_FROM_ASSET:
            if self.from_asset is None:
                raise ValueError("from_asset should not be None.")
            self.from_asset.pack(packer)
            return

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> ContractIDPreimage:
        type = ContractIDPreimageType.unpack(unpacker)
        if type == ContractIDPreimageType.CONTRACT_ID_PREIMAGE_FROM_ADDRESS:
            from_address = ContractIDPreimageFromAddress.unpack(unpacker)
            return cls(type=type, from_address=from_address)
        if type == ContractIDPreimageType.CONTRACT_ID_PREIMAGE_FROM_ASSET:
            from_asset = Asset.unpack(unpacker)
            return cls(type=type, from_asset=from_asset)
        return cls(type=type)

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> ContractIDPreimage:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> ContractIDPreimage:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.type,
                self.from_address,
                self.from_asset,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.type == other.type
            and self.from_address == other.from_address
            and self.from_asset == other.from_asset
        )

    def __repr__(self):
        out = []
        out.append(f"type={self.type}")
        (
            out.append(f"from_address={self.from_address}")
            if self.from_address is not None
            else None
        )
        (
            out.append(f"from_asset={self.from_asset}")
            if self.from_asset is not None
            else None
        )
        return f"<ContractIDPreimage [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .change_trust_asset import ChangeTrustAsset
from .int64 import Int64

__all__ = ["ChangeTrustOp"]


class ChangeTrustOp:
    """
    XDR Source Code::

        struct ChangeTrustOp
        {
            ChangeTrustAsset line;

            // if limit is set to 0, deletes the trust line
            int64 limit;
        };
    """

    def __init__(
        self,
        line: ChangeTrustAsset,
        limit: Int64,
    ) -> None:
        self.line = line
        self.limit = limit

    def pack(self, packer: Packer) -> None:
        self.line.pack(packer)
        self.limit.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> ChangeTrustOp:
        line = ChangeTrustAsset.unpack(unpacker)
        limit = Int64.unpack(unpacker)
        return cls(
            line=line,
            limit=limit,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> ChangeTrustOp:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> ChangeTrustOp:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.line,
                self.limit,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.line == other.line and self.limit == other.limit

    def __repr__(self):
        out = [
            f"line={self.line}",
            f"limit={self.limit}",
        ]
        return f"<ChangeTrustOp [{', '.join(out)}]>"



from decimal import Decimal
from typing import List, Union

from ...asset import Asset
from ...call_builder.base import BaseStrictReceivePathsCallBuilder
from ...call_builder.call_builder_sync.base_call_builder import BaseCallBuilder
from ...client.base_sync_client import BaseSyncClient

__all__ = ["StrictReceivePathsCallBuilder"]


class StrictReceivePathsCallBuilder(BaseCallBuilder, BaseStrictReceivePathsCallBuilder):
    """Creates a new :class:`StrictReceivePathsCallBuilder` pointed to server defined by horizon_url.
    Do not create this object directly, use :func:`stellar_sdk.Server.strict_receive_paths`.

    The Stellar Network allows payments to be made across assets through path payments. A path payment specifies a
    series of assets to route a payment through, from source asset (the asset debited from the payer) to destination
    asset (the asset credited to the payee).

    A path search is specified using:

    - The source address or source assets.
    - The asset and amount that the destination account should receive.

    As part of the search, horizon will load a list of assets available to the
    source address and will find any payment paths from those source assets to
    the desired destination asset. The search's amount parameter will be used to
    determine if there a given path can satisfy a payment of the desired amount.

    If a list of assets is passed as the source, horizon will find any payment
    paths from those source assets to the desired destination asset.

    See `List Strict Receive Payment Paths <https://developers.stellar.org/api/aggregations/paths/strict-receive/>`__ for more information.

    :param horizon_url: Horizon server URL.
    :param client: The client instance used to send request.
    :param source: The sender's account ID or a list of Assets. Any returned path must use a source that the sender can hold.
    :param destination_asset: The destination asset.
    :param destination_amount: The amount, denominated in the destination asset, that any returned path should be able to satisfy.
    """

    def __init__(
        self,
        horizon_url: str,
        client: BaseSyncClient,
        source: Union[str, List[Asset]],
        destination_asset: Asset,
        destination_amount: Union[str, Decimal],
    ) -> None:
        super().__init__(  # type: ignore[call-arg]
            horizon_url=horizon_url,
            client=client,
            source=source,
            destination_asset=destination_asset,
            destination_amount=destination_amount,
        )

    def stream(
        self,
    ):
        """This endpoint does not support streaming."""
        raise NotImplementedError("Streaming is not supported.")



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .account_id import AccountID
from .asset import Asset
from .int64 import Int64

__all__ = ["ClaimOfferAtom"]


class ClaimOfferAtom:
    """
    XDR Source Code::

        struct ClaimOfferAtom
        {
            // emitted to identify the offer
            AccountID sellerID; // Account that owns the offer
            int64 offerID;

            // amount and asset taken from the owner
            Asset assetSold;
            int64 amountSold;

            // amount and asset sent to the owner
            Asset assetBought;
            int64 amountBought;
        };
    """

    def __init__(
        self,
        seller_id: AccountID,
        offer_id: Int64,
        asset_sold: Asset,
        amount_sold: Int64,
        asset_bought: Asset,
        amount_bought: Int64,
    ) -> None:
        self.seller_id = seller_id
        self.offer_id = offer_id
        self.asset_sold = asset_sold
        self.amount_sold = amount_sold
        self.asset_bought = asset_bought
        self.amount_bought = amount_bought

    def pack(self, packer: Packer) -> None:
        self.seller_id.pack(packer)
        self.offer_id.pack(packer)
        self.asset_sold.pack(packer)
        self.amount_sold.pack(packer)
        self.asset_bought.pack(packer)
        self.amount_bought.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> ClaimOfferAtom:
        seller_id = AccountID.unpack(unpacker)
        offer_id = Int64.unpack(unpacker)
        asset_sold = Asset.unpack(unpacker)
        amount_sold = Int64.unpack(unpacker)
        asset_bought = Asset.unpack(unpacker)
        amount_bought = Int64.unpack(unpacker)
        return cls(
            seller_id=seller_id,
            offer_id=offer_id,
            asset_sold=asset_sold,
            amount_sold=amount_sold,
            asset_bought=asset_bought,
            amount_bought=amount_bought,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> ClaimOfferAtom:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> ClaimOfferAtom:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.seller_id,
                self.offer_id,
                self.asset_sold,
                self.amount_sold,
                self.asset_bought,
                self.amount_bought,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.seller_id == other.seller_id
            and self.offer_id == other.offer_id
            and self.asset_sold == other.asset_sold
            and self.amount_sold == other.amount_sold
            and self.asset_bought == other.asset_bought
            and self.amount_bought == other.amount_bought
        )

    def __repr__(self):
        out = [
            f"seller_id={self.seller_id}",
            f"offer_id={self.offer_id}",
            f"asset_sold={self.asset_sold}",
            f"amount_sold={self.amount_sold}",
            f"asset_bought={self.asset_bought}",
            f"amount_bought={self.amount_bought}",
        ]
        return f"<ClaimOfferAtom [{', '.join(out)}]>"



from datetime import datetime
from enum import Enum
from typing import Any, Dict, Generic, List, Optional, Sequence, TypeVar, Union

from pydantic import BaseModel, ConfigDict, Field

T = TypeVar("T")

Id = Union[str, int]


# JSON-RPC 2.0 definitions
class Request(BaseModel, Generic[T]):
    """Represent the request sent to Soroban-RPC.

    See `JSON-RPC 2.0 Specification - Request object <https://www.jsonrpc.org/specification#request_object>`__ for more information.
    """

    jsonrpc: str = "2.0"
    id: Id
    method: str
    params: Optional[T] = None


class Error(BaseModel):
    code: int
    message: Optional[str] = None
    data: Optional[str] = None


class Response(BaseModel, Generic[T]):
    """Represent the response returned from Soroban-RPC.

    See `JSON-RPC 2.0 Specification - Response object <https://www.jsonrpc.org/specification#response_object>`__ for more information.
    """

    jsonrpc: str
    id: Id
    result: Optional[T] = None
    error: Optional[Error] = None


# get_events
class EventFilterType(Enum):
    SYSTEM = "system"
    CONTRACT = "contract"
    DIAGNOSTIC = "diagnostic"


class EventFilter(BaseModel):
    event_type: Optional[EventFilterType] = Field(alias="type", default=None)
    contract_ids: Optional[Sequence[str]] = Field(alias="contractIds", default=None)
    topics: Optional[Sequence[Sequence[str]]] = None
    model_config = ConfigDict(populate_by_name=True)


class EventInfo(BaseModel):
    event_type: str = Field(alias="type")
    ledger: int = Field(alias="ledger")
    ledger_close_at: datetime = Field(alias="ledgerClosedAt")
    contract_id: str = Field(alias="contractId")
    id: str = Field(alias="id")
    paging_token: str = Field(
        alias="pagingToken",
        description="The field may will be removed in the next version of the protocol. It remains for backward.",
    )
    topic: List[str] = Field(alias="topic")
    value: str = Field(alias="value")
    in_successful_contract_call: bool = Field(alias="inSuccessfulContractCall")
    transaction_hash: str = Field(alias="txHash")


class PaginationOptions(BaseModel):
    cursor: Optional[str] = None
    limit: Optional[int] = None


class GetEventsRequest(BaseModel):
    """Response for JSON-RPC method getEvents.

    See `getEvents documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/getEvents>`__ for
    more information.
    """

    start_ledger: int = Field(alias="startLedger")
    filters: Optional[Sequence[EventFilter]] = None
    pagination: Optional[PaginationOptions] = None


class GetEventsResponse(BaseModel):
    """Response for JSON-RPC method getEvents.

    See `getEvents documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/getEvents>`__ for
    more information.
    """

    events: List[EventInfo] = Field(alias="events")
    latest_ledger: int = Field(alias="latestLedger")
    cursor: str


# get_ledger_entries
class GetLedgerEntriesRequest(BaseModel):
    """Response for JSON-RPC method getLedgerEntries.

    See `getLedgerEntries documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/getLedgerEntries>`__ for
    more information."""

    keys: Sequence[str]


class LedgerEntryResult(BaseModel):
    key: str
    xdr: str
    last_modified_ledger: int = Field(alias="lastModifiedLedgerSeq")
    live_until_ledger: Optional[int] = Field(alias="liveUntilLedgerSeq", default=None)


class GetLedgerEntriesResponse(BaseModel):
    """Response for JSON-RPC method getLedgerEntries.

    See `getLedgerEntries documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/getLedgerEntries>`__ for
    more information."""

    entries: Optional[List[LedgerEntryResult]] = None
    latest_ledger: int = Field(alias="latestLedger")


# get_network
class GetNetworkResponse(BaseModel):
    """Response for JSON-RPC method getNetwork.

    See `getNetwork documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/getNetwork>`__ for
    more information."""

    friendbot_url: Optional[str] = Field(alias="friendbotUrl", default=None)
    passphrase: str
    protocol_version: int = Field(alias="protocolVersion")


# health
class GetHealthResponse(BaseModel):
    """Response for JSON-RPC method getHealth.

    See `getHealth documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/getHealth>`__ for
    more information.
    """

    status: str
    latest_ledger: int = Field(alias="latestLedger")
    oldest_ledger: int = Field(alias="oldestLedger")
    ledger_retention_window: int = Field(alias="ledgerRetentionWindow")


# simulate_transaction
class ResourceConfig(BaseModel):
    """ResourceConfig represents the additional resource leeways for transaction simulation."""

    instruction_lee_way: int = Field(alias="instructionLeeway")
    model_config = ConfigDict(populate_by_name=True)


class SimulateTransactionRequest(BaseModel):
    """Response for JSON-RPC method simulateTransaction.

    .. note::
        The simulation response will have different model representations with different
        members present or absent depending on type of response that it is conveying. For example, the
        simulation response for invoke host function, could be one of three types: error, success, or
        restore operation needed.

    See `simulateTransaction documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/simulateTransaction>`__ for
    more information.
    """

    transaction: str
    resource_config: Optional[ResourceConfig] = Field(
        alias="resourceConfig", default=None
    )
    model_config = ConfigDict(populate_by_name=True)


class SimulateTransactionCost(BaseModel):
    cpu_insns: int = Field(alias="cpuInsns")
    mem_bytes: int = Field(alias="memBytes")


class SimulateTransactionResult(BaseModel):
    auth: Optional[List[str]] = None
    events: Optional[List[str]] = None
    footprint: str
    xdr: str


class SimulateHostFunctionResult(BaseModel):
    auth: Optional[List[str]] = None
    xdr: str


class RestorePreamble(BaseModel):
    transaction_data: str = Field(alias="transactionData")
    min_resource_fee: int = Field(alias="minResourceFee")


class LedgerEntryChange(BaseModel):
    """LedgerEntryChange designates a change in a ledger entry. Before and After cannot be omitted at the same time.
    If Before is omitted, it constitutes a creation, if After is omitted, it constitutes a deletion.
    """

    # LedgerEntryChangeType
    type: str
    # LedgerEntryKey in base64
    key: str
    # LedgerEntry XDR in base64
    before: Optional[str] = None
    # LedgerEntry XDR in base64
    after: Optional[str] = None


class SimulateTransactionResponse(BaseModel):
    """Response for JSON-RPC method simulateTransaction.

    See `simulateTransaction documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/simulateTransaction>`__ for
    more information."""

    error: Optional[str] = None
    transaction_data: Optional[str] = Field(alias="transactionData", default=None)
    # SorobanTransactionData XDR in base64
    min_resource_fee: Optional[int] = Field(alias="minResourceFee", default=None)
    events: Optional[List[str]] = None
    # DiagnosticEvent XDR in base64
    results: Optional[List[SimulateHostFunctionResult]] = None
    # the effective cpu and memory cost of the invoked transaction execution.
    restore_preamble: Optional[RestorePreamble] = Field(
        alias="restorePreamble", default=None
    )
    # If present, it indicates how the state (ledger entries) will change as a result of the transaction execution.
    state_changes: Optional[List[LedgerEntryChange]] = Field(
        alias="stateChanges", default=None
    )
    # If present, it indicates that a prior RestoreFootprint is required
    latest_ledger: int = Field(alias="latestLedger")


# get_transaction_status
class GetTransactionStatus(Enum):
    SUCCESS = "SUCCESS"
    """indicates the transaction was included in the ledger and it was executed without errors."""
    NOT_FOUND = "NOT_FOUND"
    """indicates the transaction was not found in Soroban-RPC's transaction store."""
    FAILED = "FAILED"
    """TransactionStatusFailed indicates the transaction was included in the ledger and it was executed with an error."""


class TransactionResponseError(BaseModel):
    code: str
    message: str
    data: Dict[str, Any]


class GetTransactionRequest(BaseModel):
    """Response for JSON-RPC method getTransaction.

    See `getTransaction documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/getTransaction>`__ for
    more information."""

    hash: str


class GetTransactionResponse(BaseModel):
    """Response for JSON-RPC method getTransaction.

    See `getTransaction documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/getTransaction>`__ for
    more information."""

    status: GetTransactionStatus
    transaction_hash: str = Field(alias="txHash")
    latest_ledger: int = Field(alias="latestLedger")
    latest_ledger_close_time: int = Field(alias="latestLedgerCloseTime")
    oldest_ledger: int = Field(alias="oldestLedger")
    oldest_ledger_close_time: int = Field(alias="oldestLedgerCloseTime")
    # The fields below are only present if Status is not TransactionStatus.NOT_FOUND.
    application_order: Optional[int] = Field(alias="applicationOrder", default=None)
    fee_bump: Optional[bool] = Field(alias="feeBump", default=None)
    envelope_xdr: Optional[str] = Field(
        alias="envelopeXdr", default=None
    )  # stellar_sdk.xdr.TransactionEnvelope
    result_xdr: Optional[str] = Field(
        alias="resultXdr", default=None
    )  # stellar_sdk.xdr.TransactionResult
    result_meta_xdr: Optional[str] = Field(
        alias="resultMetaXdr", default=None
    )  # stellar_sdk.xdr.TransactionMeta
    ledger: Optional[int] = Field(alias="ledger", default=None)
    create_at: Optional[int] = Field(alias="createdAt", default=None)


# send_transaction
class SendTransactionStatus(Enum):
    ERROR = "ERROR"
    """represents the status value returned by stellar-core when an error occurred from submitting a transaction"""
    PENDING = "PENDING"
    """represents the status value returned by stellar-core when a transaction has been accepted for processing"""
    DUPLICATE = "DUPLICATE"
    """represents the status value returned by stellar-core when a submitted transaction is a duplicate"""
    TRY_AGAIN_LATER = "TRY_AGAIN_LATER"
    """represents the status value returned by stellar-core when a submitted transaction was not included in the previous 4 ledgers and get banned for being added in the next few ledgers."""


class SendTransactionRequest(BaseModel):
    """Response for JSON-RPC method sendTransaction.

    See `sendTransaction documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/sendTransaction>`__ for
    more information."""

    transaction: str


class SendTransactionResponse(BaseModel):
    """Response for JSON-RPC method sendTransaction.

    See `sendTransaction documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/sendTransaction>`__ for
    more information."""

    error_result_xdr: Optional[str] = Field(alias="errorResultXdr", default=None)
    diagnostic_events_xdr: Optional[List[str]] = Field(
        alias="diagnosticEventsXdr", default=None
    )
    status: SendTransactionStatus = Field(alias="status")
    hash: str = Field(alias="hash")
    latest_ledger: int = Field(alias="latestLedger")
    latest_ledger_close_time: int = Field(alias="latestLedgerCloseTime")


# get_latest_ledger
class GetLatestLedgerResponse(BaseModel):
    """Response for JSON-RPC method getLatestLedger.

    See `getLatestLedger documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/getLatestLedger>`__ for
    more information."""

    id: str
    protocol_version: int = Field(alias="protocolVersion")
    sequence: int


# get_fee_stats
class FeeDistribution(BaseModel):
    max: int
    min: int
    mode: int
    p10: int
    p20: int
    p30: int
    p40: int
    p50: int
    p60: int
    p70: int
    p80: int
    p90: int
    p95: int
    p99: int
    transaction_count: int = Field(alias="transactionCount")
    ledger_count: int = Field(alias="ledgerCount")


class GetFeeStatsResponse(BaseModel):
    """Response for JSON-RPC method getFeeStats.

    See `getFeeStats documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/getFeeStats>`__ for
    more information."""

    soroban_inclusion_fee: FeeDistribution = Field(alias="sorobanInclusionFee")
    inclusion_fee: FeeDistribution = Field(alias="inclusionFee")
    latest_ledger: int = Field(alias="latestLedger")


# get_transactions
class GetTransactionsRequest(BaseModel):
    """Request for JSON-RPC method getTransactions.

    See `getTransactions documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/getTransactions>`__ for
    more information."""

    start_ledger: int = Field(alias="startLedger")
    pagination: Optional[PaginationOptions] = None


class Transaction(BaseModel):
    status: str
    transaction_hash: str = Field(alias="txHash")
    application_order: int = Field(alias="applicationOrder")
    fee_bump: bool = Field(alias="feeBump")
    envelope_xdr: str = Field(alias="envelopeXdr")
    result_xdr: str = Field(alias="resultXdr")
    result_meta_xdr: str = Field(alias="resultMetaXdr")
    ledger: int
    created_at: int = Field(alias="createdAt")
    diagnostic_events_xdr: Optional[List[str]] = Field(
        alias="diagnosticEventsXdr", default=None
    )


class GetTransactionsResponse(BaseModel):
    """Response for JSON-RPC method getTransactions.

    See `getTransactions documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/getTransactions>`__ for
    more information."""

    transactions: List[Transaction]
    latest_ledger: int = Field(alias="latestLedger")
    latest_ledger_close_timestamp: int = Field(alias="latestLedgerCloseTimestamp")
    oldest_ledger: int = Field(alias="oldestLedger")
    oldest_ledger_close_timestamp: int = Field(alias="oldestLedgerCloseTimestamp")
    cursor: str


# get_version_info
class GetVersionInfoResponse(BaseModel):
    """Response for JSON-RPC method getVersionInfo.

    See `getVersionInfo documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/getVersionInfo>`__ for
    more information."""

    version: str
    commit_hash: str = Field(alias="commitHash")
    build_timestamp: str = Field(alias="buildTimestamp")
    captive_core_version: str = Field(alias="captiveCoreVersion")
    protocol_version: int = Field(alias="protocolVersion")


# get_ledgers
class GetLedgersRequest(BaseModel):
    """Request for JSON-RPC method getLedgers.

    See `getLedgers documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/getLedgers>`__ for
    more information."""

    start_ledger: int = Field(alias="startLedger")
    pagination: Optional[PaginationOptions] = None


class LedgerInfo(BaseModel):
    hash: str
    sequence: int
    ledger_close_time: int = Field(alias="ledgerCloseTime")
    # LedgerHeaderHistoryEntry XDR in base64
    header_xdr: str = Field(alias="headerXdr")
    # LedgerCloseMeta XDR in base64
    metadata_xdr: str = Field(alias="metadataXdr")


class GetLedgersResponse(BaseModel):
    """Response for JSON-RPC method getLedgers.

    See `getLedgers documentation <https://developers.stellar.org/docs/data/rpc/api-reference/methods/getLedgers>`__ for
    more information."""

    ledgers: List[LedgerInfo]
    latest_ledger: int = Field(alias="latestLedger")
    latest_ledger_close_time: int = Field(alias="latestLedgerCloseTime")
    oldest_ledger: int = Field(alias="oldestLedger")
    oldest_ledger_close_time: int = Field(alias="oldestLedgerCloseTime")
    cursor: str



# Automatically generated by xdrgen
# DO NOT EDIT or your changes may be overwritten
from .account_entry import *
from .account_entry_ext import *
from .account_entry_extension_v1 import *
from .account_entry_extension_v1_ext import *
from .account_entry_extension_v2 import *
from .account_entry_extension_v2_ext import *
from .account_entry_extension_v3 import *
from .account_flags import *
from .account_id import *
from .account_merge_result import *
from .account_merge_result_code import *
from .allow_trust_op import *
from .allow_trust_result import *
from .allow_trust_result_code import *
from .alpha_num4 import *
from .alpha_num12 import *
from .archival_proof import *
from .archival_proof_body import *
from .archival_proof_node import *
from .archival_proof_type import *
from .asset import *
from .asset_code import *
from .asset_code4 import *
from .asset_code12 import *
from .asset_type import *
from .auth import *
from .auth_cert import *
from .authenticated_message import *
from .authenticated_message_v0 import *
from .base import *
from .begin_sponsoring_future_reserves_op import *
from .begin_sponsoring_future_reserves_result import *
from .begin_sponsoring_future_reserves_result_code import *
from .binary_fuse_filter_type import *
from .bucket_entry import *
from .bucket_entry_type import *
from .bucket_list_type import *
from .bucket_metadata import *
from .bucket_metadata_ext import *
from .bump_sequence_op import *
from .bump_sequence_result import *
from .bump_sequence_result_code import *
from .change_trust_asset import *
from .change_trust_op import *
from .change_trust_result import *
from .change_trust_result_code import *
from .claim_atom import *
from .claim_atom_type import *
from .claim_claimable_balance_op import *
from .claim_claimable_balance_result import *
from .claim_claimable_balance_result_code import *
from .claim_liquidity_atom import *
from .claim_offer_atom import *
from .claim_offer_atom_v0 import *
from .claim_predicate import *
from .claim_predicate_type import *
from .claimable_balance_entry import *
from .claimable_balance_entry_ext import *
from .claimable_balance_entry_extension_v1 import *
from .claimable_balance_entry_extension_v1_ext import *
from .claimable_balance_flags import *
from .claimable_balance_id import *
from .claimable_balance_id_type import *
from .claimant import *
from .claimant_type import *
from .claimant_v0 import *
from .clawback_claimable_balance_op import *
from .clawback_claimable_balance_result import *
from .clawback_claimable_balance_result_code import *
from .clawback_op import *
from .clawback_result import *
from .clawback_result_code import *
from .cold_archive_archived_leaf import *
from .cold_archive_boundary_leaf import *
from .cold_archive_bucket_entry import *
from .cold_archive_bucket_entry_type import *
from .cold_archive_deleted_leaf import *
from .cold_archive_hash_entry import *
from .config_setting_contract_bandwidth_v0 import *
from .config_setting_contract_compute_v0 import *
from .config_setting_contract_events_v0 import *
from .config_setting_contract_execution_lanes_v0 import *
from .config_setting_contract_historical_data_v0 import *
from .config_setting_contract_ledger_cost_v0 import *
from .config_setting_entry import *
from .config_setting_id import *
from .config_upgrade_set import *
from .config_upgrade_set_key import *
from .constants import *
from .contract_code_cost_inputs import *
from .contract_code_entry import *
from .contract_code_entry_ext import *
from .contract_code_entry_v1 import *
from .contract_cost_param_entry import *
from .contract_cost_params import *
from .contract_cost_type import *
from .contract_data_durability import *
from .contract_data_entry import *
from .contract_event import *
from .contract_event_body import *
from .contract_event_type import *
from .contract_event_v0 import *
from .contract_executable import *
from .contract_executable_type import *
from .contract_id_preimage import *
from .contract_id_preimage_from_address import *
from .contract_id_preimage_type import *
from .create_account_op import *
from .create_account_result import *
from .create_account_result_code import *
from .create_claimable_balance_op import *
from .create_claimable_balance_result import *
from .create_claimable_balance_result_code import *
from .create_contract_args import *
from .create_contract_args_v2 import *
from .create_passive_sell_offer_op import *
from .crypto_key_type import *
from .curve25519_public import *
from .curve25519_secret import *
from .data_entry import *
from .data_entry_ext import *
from .data_value import *
from .decorated_signature import *
from .diagnostic_event import *
from .diagnostic_events import *
from .dont_have import *
from .duration import *
from .encrypted_body import *
from .end_sponsoring_future_reserves_result import *
from .end_sponsoring_future_reserves_result_code import *
from .envelope_type import *
from .error import *
from .error_code import *
from .eviction_iterator import *
from .existence_proof_body import *
from .extend_footprint_ttl_op import *
from .extend_footprint_ttl_result import *
from .extend_footprint_ttl_result_code import *
from .extension_point import *
from .fee_bump_transaction import *
from .fee_bump_transaction_envelope import *
from .fee_bump_transaction_ext import *
from .fee_bump_transaction_inner_tx import *
from .flood_advert import *
from .flood_demand import *
from .generalized_transaction_set import *
from .hash import *
from .hash_id_preimage import *
from .hash_id_preimage_contract_id import *
from .hash_id_preimage_operation_id import *
from .hash_id_preimage_revoke_id import *
from .hash_id_preimage_soroban_authorization import *
from .hello import *
from .hmac_sha256_key import *
from .hmac_sha256_mac import *
from .host_function import *
from .host_function_type import *
from .hot_archive_bucket_entry import *
from .hot_archive_bucket_entry_type import *
from .inflation_payout import *
from .inflation_result import *
from .inflation_result_code import *
from .inner_transaction_result import *
from .inner_transaction_result_ext import *
from .inner_transaction_result_pair import *
from .inner_transaction_result_result import *
from .int32 import *
from .int64 import *
from .int128_parts import *
from .int256_parts import *
from .invoke_contract_args import *
from .invoke_host_function_op import *
from .invoke_host_function_result import *
from .invoke_host_function_result_code import *
from .invoke_host_function_success_pre_image import *
from .ip_addr_type import *
from .ledger_bounds import *
from .ledger_close_meta import *
from .ledger_close_meta_ext import *
from .ledger_close_meta_ext_v1 import *
from .ledger_close_meta_v0 import *
from .ledger_close_meta_v1 import *
from .ledger_close_value_signature import *
from .ledger_entry import *
from .ledger_entry_change import *
from .ledger_entry_change_type import *
from .ledger_entry_changes import *
from .ledger_entry_data import *
from .ledger_entry_ext import *
from .ledger_entry_extension_v1 import *
from .ledger_entry_extension_v1_ext import *
from .ledger_entry_type import *
from .ledger_footprint import *
from .ledger_header import *
from .ledger_header_ext import *
from .ledger_header_extension_v1 import *
from .ledger_header_extension_v1_ext import *
from .ledger_header_flags import *
from .ledger_header_history_entry import *
from .ledger_header_history_entry_ext import *
from .ledger_key import *
from .ledger_key_account import *
from .ledger_key_claimable_balance import *
from .ledger_key_config_setting import *
from .ledger_key_contract_code import *
from .ledger_key_contract_data import *
from .ledger_key_data import *
from .ledger_key_liquidity_pool import *
from .ledger_key_offer import *
from .ledger_key_trust_line import *
from .ledger_key_ttl import *
from .ledger_scp_messages import *
from .ledger_upgrade import *
from .ledger_upgrade_type import *
from .liabilities import *
from .liquidity_pool_constant_product_parameters import *
from .liquidity_pool_deposit_op import *
from .liquidity_pool_deposit_result import *
from .liquidity_pool_deposit_result_code import *
from .liquidity_pool_entry import *
from .liquidity_pool_entry_body import *
from .liquidity_pool_entry_constant_product import *
from .liquidity_pool_parameters import *
from .liquidity_pool_type import *
from .liquidity_pool_withdraw_op import *
from .liquidity_pool_withdraw_result import *
from .liquidity_pool_withdraw_result_code import *
from .manage_buy_offer_op import *
from .manage_buy_offer_result import *
from .manage_buy_offer_result_code import *
from .manage_data_op import *
from .manage_data_result import *
from .manage_data_result_code import *
from .manage_offer_effect import *
from .manage_offer_success_result import *
from .manage_offer_success_result_offer import *
from .manage_sell_offer_op import *
from .manage_sell_offer_result import *
from .manage_sell_offer_result_code import *
from .memo import *
from .memo_type import *
from .message_type import *
from .muxed_account import *
from .muxed_account_med25519 import *
from .node_id import *
from .nonexistence_proof_body import *
from .offer_entry import *
from .offer_entry_ext import *
from .offer_entry_flags import *
from .operation import *
from .operation_body import *
from .operation_meta import *
from .operation_result import *
from .operation_result_code import *
from .operation_result_tr import *
from .operation_type import *
from .path_payment_strict_receive_op import *
from .path_payment_strict_receive_result import *
from .path_payment_strict_receive_result_code import *
from .path_payment_strict_receive_result_success import *
from .path_payment_strict_send_op import *
from .path_payment_strict_send_result import *
from .path_payment_strict_send_result_code import *
from .path_payment_strict_send_result_success import *
from .payment_op import *
from .payment_result import *
from .payment_result_code import *
from .peer_address import *
from .peer_address_ip import *
from .peer_stat_list import *
from .peer_stats import *
from .persisted_scp_state import *
from .persisted_scp_state_v0 import *
from .persisted_scp_state_v1 import *
from .pool_id import *
from .precondition_type import *
from .preconditions import *
from .preconditions_v2 import *
from .price import *
from .proof_level import *
from .public_key import *
from .public_key_type import *
from .restore_footprint_op import *
from .restore_footprint_result import *
from .restore_footprint_result_code import *
from .revoke_sponsorship_op import *
from .revoke_sponsorship_op_signer import *
from .revoke_sponsorship_result import *
from .revoke_sponsorship_result_code import *
from .revoke_sponsorship_type import *
from .sc_address import *
from .sc_address_type import *
from .sc_bytes import *
from .sc_contract_instance import *
from .sc_env_meta_entry import *
from .sc_env_meta_entry_interface_version import *
from .sc_env_meta_kind import *
from .sc_error import *
from .sc_error_code import *
from .sc_error_type import *
from .sc_map import *
from .sc_map_entry import *
from .sc_meta_entry import *
from .sc_meta_kind import *
from .sc_meta_v0 import *
from .sc_nonce_key import *
from .sc_spec_entry import *
from .sc_spec_entry_kind import *
from .sc_spec_function_input_v0 import *
from .sc_spec_function_v0 import *
from .sc_spec_type import *
from .sc_spec_type_bytes_n import *
from .sc_spec_type_def import *
from .sc_spec_type_map import *
from .sc_spec_type_option import *
from .sc_spec_type_result import *
from .sc_spec_type_tuple import *
from .sc_spec_type_udt import *
from .sc_spec_type_vec import *
from .sc_spec_udt_enum_case_v0 import *
from .sc_spec_udt_enum_v0 import *
from .sc_spec_udt_error_enum_case_v0 import *
from .sc_spec_udt_error_enum_v0 import *
from .sc_spec_udt_struct_field_v0 import *
from .sc_spec_udt_struct_v0 import *
from .sc_spec_udt_union_case_tuple_v0 import *
from .sc_spec_udt_union_case_v0 import *
from .sc_spec_udt_union_case_v0_kind import *
from .sc_spec_udt_union_case_void_v0 import *
from .sc_spec_udt_union_v0 import *
from .sc_string import *
from .sc_symbol import *
from .sc_val import *
from .sc_val_type import *
from .sc_vec import *
from .scp_ballot import *
from .scp_envelope import *
from .scp_history_entry import *
from .scp_history_entry_v0 import *
from .scp_nomination import *
from .scp_quorum_set import *
from .scp_statement import *
from .scp_statement_confirm import *
from .scp_statement_externalize import *
from .scp_statement_pledges import *
from .scp_statement_prepare import *
from .scp_statement_type import *
from .send_more import *
from .send_more_extended import *
from .sequence_number import *
from .serialized_binary_fuse_filter import *
from .set_options_op import *
from .set_options_result import *
from .set_options_result_code import *
from .set_trust_line_flags_op import *
from .set_trust_line_flags_result import *
from .set_trust_line_flags_result_code import *
from .short_hash_seed import *
from .signature import *
from .signature_hint import *
from .signed_survey_request_message import *
from .signed_survey_response_message import *
from .signed_time_sliced_survey_request_message import *
from .signed_time_sliced_survey_response_message import *
from .signed_time_sliced_survey_start_collecting_message import *
from .signed_time_sliced_survey_stop_collecting_message import *
from .signer import *
from .signer_key import *
from .signer_key_ed25519_signed_payload import *
from .signer_key_type import *
from .simple_payment_result import *
from .soroban_address_credentials import *
from .soroban_authorization_entry import *
from .soroban_authorized_function import *
from .soroban_authorized_function_type import *
from .soroban_authorized_invocation import *
from .soroban_credentials import *
from .soroban_credentials_type import *
from .soroban_resources import *
from .soroban_transaction_data import *
from .soroban_transaction_meta import *
from .soroban_transaction_meta_ext import *
from .soroban_transaction_meta_ext_v1 import *
from .sponsorship_descriptor import *
from .state_archival_settings import *
from .stellar_message import *
from .stellar_value import *
from .stellar_value_ext import *
from .stellar_value_type import *
from .stored_debug_transaction_set import *
from .stored_transaction_set import *
from .string32 import *
from .string64 import *
from .survey_message_command_type import *
from .survey_message_response_type import *
from .survey_request_message import *
from .survey_response_body import *
from .survey_response_message import *
from .threshold_indexes import *
from .thresholds import *
from .time_bounds import *
from .time_point import *
from .time_sliced_node_data import *
from .time_sliced_peer_data import *
from .time_sliced_peer_data_list import *
from .time_sliced_survey_request_message import *
from .time_sliced_survey_response_message import *
from .time_sliced_survey_start_collecting_message import *
from .time_sliced_survey_stop_collecting_message import *
from .topology_response_body_v0 import *
from .topology_response_body_v1 import *
from .topology_response_body_v2 import *
from .transaction import *
from .transaction_envelope import *
from .transaction_ext import *
from .transaction_history_entry import *
from .transaction_history_entry_ext import *
from .transaction_history_result_entry import *
from .transaction_history_result_entry_ext import *
from .transaction_meta import *
from .transaction_meta_v1 import *
from .transaction_meta_v2 import *
from .transaction_meta_v3 import *
from .transaction_phase import *
from .transaction_result import *
from .transaction_result_code import *
from .transaction_result_ext import *
from .transaction_result_meta import *
from .transaction_result_pair import *
from .transaction_result_result import *
from .transaction_result_set import *
from .transaction_set import *
from .transaction_set_v1 import *
from .transaction_signature_payload import *
from .transaction_signature_payload_tagged_transaction import *
from .transaction_v0 import *
from .transaction_v0_envelope import *
from .transaction_v0_ext import *
from .transaction_v1_envelope import *
from .trust_line_asset import *
from .trust_line_entry import *
from .trust_line_entry_ext import *
from .trust_line_entry_extension_v2 import *
from .trust_line_entry_extension_v2_ext import *
from .trust_line_entry_v1 import *
from .trust_line_entry_v1_ext import *
from .trust_line_flags import *
from .ttl_entry import *
from .tx_advert_vector import *
from .tx_demand_vector import *
from .tx_set_component import *
from .tx_set_component_txs_maybe_discounted_fee import *
from .tx_set_component_type import *
from .u_int128_parts import *
from .u_int256_parts import *
from .uint32 import *
from .uint64 import *
from .uint256 import *
from .upgrade_entry_meta import *
from .upgrade_type import *
from .value import *



"""
SEP: 0035
Title: Operation IDs
Author: Scott Fleckenstein (@nullstyle), Isaiah Turner (@Isaiah-Turner), Debnil Sur (@debnil)
Track: Standard
Status: Draft
Created: 2020-08-26
Discussion: https://groups.google.com/g/stellar-dev/c/vCgQhmox32Q
"""

__all__ = ["TOID"]

from typing import Tuple


class TOID:
    """TOID represents the total order of Ledgers, Transactions and Operations.
    This is an implementation of SEP-35: https://github.com/stellar/stellar-protocol/blob/master/ecosystem/sep-0035.md

    Operations within the stellar network have a total order, expressed by three
    pieces of information: the ledger sequence the operation was validated in,
    the order which the operation's containing transaction was applied in
    that ledger, and the index of the operation within that parent transaction.

    :param ledger_sequence: The ledger sequence the operation was validated in.
    :param transaction_order: The order that the transaction was applied within the ledger where it was validated.
        The application order value starts at 1. The maximum supported number of transactions
        per operation is 1,048,575.
    :param operation_order: The index of the operation within that parent transaction. The operation index value
        starts at 1. The maximum supported number of operations per transaction is 4095.
    """

    def __init__(
        self, ledger_sequence: int, transaction_order: int, operation_order: int
    ) -> None:
        if ledger_sequence < 0 or ledger_sequence > 2**31 - 1:
            raise ValueError(
                "Invalid `ledger_sequence`, it must be between 0 and 2147483647."
            )

        if transaction_order < 0 or transaction_order > 2**20 - 1:
            raise ValueError(
                "Invalid `transaction_order`, it must be between 0 and 1048575."
            )

        if operation_order < 0 or operation_order > 2**12 - 1:
            raise ValueError(
                "Invalid `operation_order`, it must be between 0 and 4095."
            )

        self.ledger_sequence = ledger_sequence
        self.transaction_order = transaction_order
        self.operation_order = operation_order

    def to_int64(self) -> int:
        """Converts the TOID to a signed 64-bit integer.

        :return: The signed 64-bit integer representation of the TOID.
        """
        return (
            self.ledger_sequence << 32
            | self.transaction_order << 12
            | self.operation_order
        )

    @classmethod
    def from_int64(cls, value: int) -> "TOID":
        """Converts a signed 64-bit integer to a TOID.

        :param value: The signed 64-bit integer to convert.
        """
        if value < 0 or value > 2**63 - 1:
            raise ValueError(
                "Invalid `value`, it must be between 0 and 9223372036854775807."
            )
        return TOID(value >> 32, (value >> 12) & 0xFFFFF, value & 0xFFF)

    def increment_operation_order(self) -> None:
        """Increments the operation order by 1, rolling over to the next ledger if overflow occurs.
        This allows queries to easily advance a cursor to the next operation.

        :return: The current TOID instance.
        """
        if self.operation_order == 2**12 - 1:
            self.operation_order = 0
            self.ledger_sequence += 1
        else:
            self.operation_order += 1

    @classmethod
    def after_ledger(cls, ledger_sequence: int) -> "TOID":
        """Creates a new toid that represents the ledger time **after** any
        contents (e.g. transactions, operations) that occur within the specified ledger.

        :param ledger_sequence: The ledger sequence.
        :return: The TOID instance.
        """
        return TOID(ledger_sequence, 2**20 - 1, 2**12 - 1)

    @staticmethod
    def ledger_range_inclusive(start: int, end: int) -> Tuple[int, int]:
        """The inclusive range representation between two
        ledgers inclusive. The second value points at the end+1 ledger so when using
        this value make sure < order is used.

        :param start: The start ledger sequence.
        :param end: The end ledger sequence.
        :return: The inclusive range representation between two ledgers.
        """
        if start > end:
            raise ValueError(
                "Invalid `start` and `end` values, `start` must be less than or equal to `end`."
            )
        toid_start = 0
        if start != 1:
            toid_start = TOID(start, 0, 0).to_int64()
        toid_end = TOID(end + 1, 0, 0).to_int64()
        return toid_start, toid_end

    def __hash__(self):
        return hash(
            (self.ledger_sequence, self.transaction_order, self.operation_order)
        )

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.ledger_sequence == other.ledger_sequence
            and self.transaction_order == other.transaction_order
            and self.operation_order == other.operation_order
        )

    def __repr__(self) -> str:
        return (
            f"<TOID [ledger_sequence={self.ledger_sequence}, "
            f"transaction_order={self.transaction_order}, "
            f"operation_order={self.operation_order}]>"
        )



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from typing import List

from xdrlib3 import Packer, Unpacker

from .archival_proof_node import ArchivalProofNode

__all__ = ["ProofLevel"]


class ProofLevel:
    """
    XDR Source Code::

        typedef ArchivalProofNode ProofLevel<>;
    """

    def __init__(self, proof_level: List[ArchivalProofNode]) -> None:
        _expect_max_length = 4294967295
        if proof_level and len(proof_level) > _expect_max_length:
            raise ValueError(
                f"The maximum length of `proof_level` should be {_expect_max_length}, but got {len(proof_level)}."
            )
        self.proof_level = proof_level

    def pack(self, packer: Packer) -> None:
        packer.pack_uint(len(self.proof_level))
        for proof_level_item in self.proof_level:
            proof_level_item.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> ProofLevel:
        length = unpacker.unpack_uint()
        proof_level = []
        for _ in range(length):
            proof_level.append(ArchivalProofNode.unpack(unpacker))
        return cls(proof_level)

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> ProofLevel:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> ProofLevel:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(self.proof_level)

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.proof_level == other.proof_level

    def __repr__(self):
        return f"<ProofLevel [proof_level={self.proof_level}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .survey_request_message import SurveyRequestMessage
from .uint32 import Uint32

__all__ = ["TimeSlicedSurveyRequestMessage"]


class TimeSlicedSurveyRequestMessage:
    """
    XDR Source Code::

        struct TimeSlicedSurveyRequestMessage
        {
            SurveyRequestMessage request;
            uint32 nonce;
            uint32 inboundPeersIndex;
            uint32 outboundPeersIndex;
        };
    """

    def __init__(
        self,
        request: SurveyRequestMessage,
        nonce: Uint32,
        inbound_peers_index: Uint32,
        outbound_peers_index: Uint32,
    ) -> None:
        self.request = request
        self.nonce = nonce
        self.inbound_peers_index = inbound_peers_index
        self.outbound_peers_index = outbound_peers_index

    def pack(self, packer: Packer) -> None:
        self.request.pack(packer)
        self.nonce.pack(packer)
        self.inbound_peers_index.pack(packer)
        self.outbound_peers_index.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> TimeSlicedSurveyRequestMessage:
        request = SurveyRequestMessage.unpack(unpacker)
        nonce = Uint32.unpack(unpacker)
        inbound_peers_index = Uint32.unpack(unpacker)
        outbound_peers_index = Uint32.unpack(unpacker)
        return cls(
            request=request,
            nonce=nonce,
            inbound_peers_index=inbound_peers_index,
            outbound_peers_index=outbound_peers_index,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> TimeSlicedSurveyRequestMessage:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> TimeSlicedSurveyRequestMessage:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.request,
                self.nonce,
                self.inbound_peers_index,
                self.outbound_peers_index,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.request == other.request
            and self.nonce == other.nonce
            and self.inbound_peers_index == other.inbound_peers_index
            and self.outbound_peers_index == other.outbound_peers_index
        )

    def __repr__(self):
        out = [
            f"request={self.request}",
            f"nonce={self.nonce}",
            f"inbound_peers_index={self.inbound_peers_index}",
            f"outbound_peers_index={self.outbound_peers_index}",
        ]
        return f"<TimeSlicedSurveyRequestMessage [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from enum import IntEnum

from xdrlib3 import Packer, Unpacker

__all__ = ["SCErrorType"]


class SCErrorType(IntEnum):
    """
    XDR Source Code::

        enum SCErrorType
        {
            SCE_CONTRACT = 0,          // Contract-specific, user-defined codes.
            SCE_WASM_VM = 1,           // Errors while interpreting WASM bytecode.
            SCE_CONTEXT = 2,           // Errors in the contract's host context.
            SCE_STORAGE = 3,           // Errors accessing host storage.
            SCE_OBJECT = 4,            // Errors working with host objects.
            SCE_CRYPTO = 5,            // Errors in cryptographic operations.
            SCE_EVENTS = 6,            // Errors while emitting events.
            SCE_BUDGET = 7,            // Errors relating to budget limits.
            SCE_VALUE = 8,             // Errors working with host values or SCVals.
            SCE_AUTH = 9               // Errors from the authentication subsystem.
        };
    """

    SCE_CONTRACT = 0
    SCE_WASM_VM = 1
    SCE_CONTEXT = 2
    SCE_STORAGE = 3
    SCE_OBJECT = 4
    SCE_CRYPTO = 5
    SCE_EVENTS = 6
    SCE_BUDGET = 7
    SCE_VALUE = 8
    SCE_AUTH = 9

    def pack(self, packer: Packer) -> None:
        packer.pack_int(self.value)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> SCErrorType:
        value = unpacker.unpack_int()
        return cls(value)

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> SCErrorType:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> SCErrorType:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .ledger_footprint import LedgerFootprint
from .uint32 import Uint32

__all__ = ["SorobanResources"]


class SorobanResources:
    """
    XDR Source Code::

        struct SorobanResources
        {
            // The ledger footprint of the transaction.
            LedgerFootprint footprint;
            // The maximum number of instructions this transaction can use
            uint32 instructions;

            // The maximum number of bytes this transaction can read from ledger
            uint32 readBytes;
            // The maximum number of bytes this transaction can write to ledger
            uint32 writeBytes;
        };
    """

    def __init__(
        self,
        footprint: LedgerFootprint,
        instructions: Uint32,
        read_bytes: Uint32,
        write_bytes: Uint32,
    ) -> None:
        self.footprint = footprint
        self.instructions = instructions
        self.read_bytes = read_bytes
        self.write_bytes = write_bytes

    def pack(self, packer: Packer) -> None:
        self.footprint.pack(packer)
        self.instructions.pack(packer)
        self.read_bytes.pack(packer)
        self.write_bytes.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> SorobanResources:
        footprint = LedgerFootprint.unpack(unpacker)
        instructions = Uint32.unpack(unpacker)
        read_bytes = Uint32.unpack(unpacker)
        write_bytes = Uint32.unpack(unpacker)
        return cls(
            footprint=footprint,
            instructions=instructions,
            read_bytes=read_bytes,
            write_bytes=write_bytes,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> SorobanResources:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> SorobanResources:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.footprint,
                self.instructions,
                self.read_bytes,
                self.write_bytes,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.footprint == other.footprint
            and self.instructions == other.instructions
            and self.read_bytes == other.read_bytes
            and self.write_bytes == other.write_bytes
        )

    def __repr__(self):
        out = [
            f"footprint={self.footprint}",
            f"instructions={self.instructions}",
            f"read_bytes={self.read_bytes}",
            f"write_bytes={self.write_bytes}",
        ]
        return f"<SorobanResources [{', '.join(out)}]>"



from __future__ import annotations

from typing import List, Union

from . import xdr as stellar_xdr

__all__ = ["SorobanDataBuilder"]


class SorobanDataBuilder:
    """Supports building :class:`Memo <stellar_sdk.xdr.SorobanTransactionData>` structures
    with various items set to specific values.

    This is recommended for when you are building :class:`RestoreFootprint <stellar_sdk.operation.RestoreFootprint>`,
    :class:`ExtendFootprintTTL <stellar_sdk.operation.ExtendFootprintTTL>` operations to avoid (re)building
    the entire data structure from scratch.

    By default, an empty instance will be created.
    """

    def __init__(self):
        self._data = stellar_xdr.SorobanTransactionData(
            ext=stellar_xdr.ExtensionPoint(0),
            resource_fee=stellar_xdr.Int64(0),
            resources=stellar_xdr.SorobanResources(
                footprint=stellar_xdr.LedgerFootprint(
                    read_only=[],
                    read_write=[],
                ),
                read_bytes=stellar_xdr.Uint32(0),
                write_bytes=stellar_xdr.Uint32(0),
                instructions=stellar_xdr.Uint32(0),
            ),
        )

    @classmethod
    def from_xdr(
        cls, soroban_data: Union[str, stellar_xdr.SorobanTransactionData]
    ) -> SorobanDataBuilder:
        """Create a new :class:`SorobanDataBuilder` object from an XDR object.

        :param soroban_data: The XDR object that represents a SorobanTransactionData.
        :return: This builder.
        """
        data = cls()
        if isinstance(soroban_data, str):
            data._data = stellar_xdr.SorobanTransactionData.from_xdr(soroban_data)
        else:
            data._data = stellar_xdr.SorobanTransactionData.from_xdr_bytes(
                soroban_data.to_xdr_bytes()
            )
        return data

    def set_resource_fee(self, fee: int) -> SorobanDataBuilder:
        """Sets the "resource" fee portion of the Soroban data.

        :param fee: The resource fee to set (int64)
        :return: This builder.
        """
        self._data.resource_fee = stellar_xdr.Int64(fee)
        return self

    def set_read_only(
        self, read_only: List[stellar_xdr.LedgerKey]
    ) -> SorobanDataBuilder:
        """Sets the read-only portion of the storage access footprint to be a certain set of ledger keys.

        :param read_only: The read-only ledger keys to set.
        :return: This builder.
        """
        self._data.resources.footprint.read_only = read_only or []
        return self

    def set_read_write(
        self, read_write: List[stellar_xdr.LedgerKey]
    ) -> SorobanDataBuilder:
        """Sets the read-write portion of the storage access footprint to be a certain set of ledger keys.

        :param read_write: The read-write ledger keys to set.
        :return: This builder.
        """
        self._data.resources.footprint.read_write = read_write or []
        return self

    def set_resources(
        self, instructions: int, read_bytes: int, write_bytes: int
    ) -> SorobanDataBuilder:
        """Sets up the resource metrics.

        You should almost NEVER need this, as its often generated / provided to you
        by transaction simulation/preflight from a Soroban RPC server.

        :param instructions: Number of CPU instructions (uint32)
        :param read_bytes: Number of bytes being read (uint32)
        :param write_bytes: Number of bytes being written (uint32)
        :return: This builder.
        """
        self._data.resources.instructions = stellar_xdr.Uint32(instructions)
        self._data.resources.read_bytes = stellar_xdr.Uint32(read_bytes)
        self._data.resources.write_bytes = stellar_xdr.Uint32(write_bytes)
        return self

    def build(self):
        """:return: a copy of the final data structure."""
        return stellar_xdr.SorobanTransactionData.from_xdr_bytes(
            self._data.to_xdr_bytes()
        )



from decimal import Decimal
from typing import Optional, Sequence, Union

from .. import xdr as stellar_xdr
from ..asset import Asset
from ..muxed_account import MuxedAccount
from ..utils import raise_if_not_valid_amount
from .operation import Operation

__all__ = ["PathPaymentStrictSend"]


class PathPaymentStrictSend(Operation):
    """The :class:`PathPaymentStrictSend` object, which represents a PathPaymentStrictSend
    operation on Stellar's network.

    Sends an amount in a specific asset to a destination account through a path
    of offers. This allows the asset sent (e.g, 450 XLM) to be different from
    the asset received (e.g, 6 BTC).

    Threshold: Medium

    See `Path Payment Strict Send <https://developers.stellar.org/docs/start/list-of-operations/#path-payment-strict-send>`_ for more information.

    :param destination: The destination account to send to.
    :param send_asset: The `asset` to pay with.
    :param send_amount: Amount of `send_asset` to send.
    :param dest_asset: The asset the `destination` will receive.
    :param dest_min: The minimum amount of `dest_asset` to be received.
    :param path: A list of Asset objects to use as the path.
    :param source: The source account for the operation. Defaults to the
        transaction's source account.
    """

    _XDR_OPERATION_TYPE: stellar_xdr.OperationType = (
        stellar_xdr.OperationType.PATH_PAYMENT_STRICT_SEND
    )

    def __init__(
        self,
        destination: Union[MuxedAccount, str],
        send_asset: Asset,
        send_amount: Union[str, Decimal],
        dest_asset: Asset,
        dest_min: Union[str, Decimal],
        path: Sequence[Asset],
        source: Optional[Union[MuxedAccount, str]] = None,
    ) -> None:
        super().__init__(source)
        if isinstance(destination, str):
            destination = MuxedAccount.from_account(destination)
        self.destination: MuxedAccount = destination
        self.send_asset: Asset = send_asset
        self.send_amount: str = str(send_amount)
        self.dest_asset: Asset = dest_asset
        self.dest_min: str = str(dest_min)
        self.path: Sequence[Asset] = path  # a list of paths/assets
        raise_if_not_valid_amount(self.send_amount, "send_amount")
        raise_if_not_valid_amount(self.dest_min, "dest_min")

    def _to_operation_body(self) -> stellar_xdr.OperationBody:
        destination = self.destination.to_xdr_object()
        send_asset = self.send_asset.to_xdr_object()
        dest_asset = self.dest_asset.to_xdr_object()
        path = [asset.to_xdr_object() for asset in self.path]

        path_payment_strict_send_op = stellar_xdr.PathPaymentStrictSendOp(
            send_asset,
            stellar_xdr.Int64(Operation.to_xdr_amount(self.send_amount)),
            destination,
            dest_asset,
            stellar_xdr.Int64(Operation.to_xdr_amount(self.dest_min)),
            path,
        )
        body = stellar_xdr.OperationBody(
            type=self._XDR_OPERATION_TYPE,
            path_payment_strict_send_op=path_payment_strict_send_op,
        )
        return body

    @classmethod
    def from_xdr_object(
        cls, xdr_object: stellar_xdr.Operation
    ) -> "PathPaymentStrictSend":
        """Creates a :class:`PathPaymentStrictSend` object from an XDR Operation
        object.

        """
        source = Operation.get_source_from_xdr_obj(xdr_object)
        assert xdr_object.body.path_payment_strict_send_op is not None
        destination = MuxedAccount.from_xdr_object(
            xdr_object.body.path_payment_strict_send_op.destination
        )
        send_asset = Asset.from_xdr_object(
            xdr_object.body.path_payment_strict_send_op.send_asset
        )
        dest_asset = Asset.from_xdr_object(
            xdr_object.body.path_payment_strict_send_op.dest_asset
        )
        send_amount = Operation.from_xdr_amount(
            xdr_object.body.path_payment_strict_send_op.send_amount.int64
        )
        dest_min = Operation.from_xdr_amount(
            xdr_object.body.path_payment_strict_send_op.dest_min.int64
        )

        path = []
        # In fact, we don't need to check it.
        if xdr_object.body.path_payment_strict_send_op.path:
            for x in xdr_object.body.path_payment_strict_send_op.path:
                path.append(Asset.from_xdr_object(x))

        op = cls(
            source=source,
            destination=destination,
            send_asset=send_asset,
            send_amount=send_amount,
            dest_asset=dest_asset,
            dest_min=dest_min,
            path=path,
        )
        return op

    def __repr__(self):
        return (
            f"<PathPaymentStrictSend [destination={self.destination}, send_asset={self.send_asset}, "
            f"send_amount={self.send_amount}, dest_asset={self.dest_asset}, dest_min={self.dest_min}, "
            f"path={self.path}, source={self.source}]>"
        )



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .base import Opaque

__all__ = ["ShortHashSeed"]


class ShortHashSeed:
    """
    XDR Source Code::

        struct ShortHashSeed
        {
            opaque seed[16];
        };
    """

    def __init__(
        self,
        seed: bytes,
    ) -> None:
        self.seed = seed

    def pack(self, packer: Packer) -> None:
        Opaque(self.seed, 16, True).pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> ShortHashSeed:
        seed = Opaque.unpack(unpacker, 16, True)
        return cls(
            seed=seed,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> ShortHashSeed:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> ShortHashSeed:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash((self.seed,))

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.seed == other.seed

    def __repr__(self):
        out = [
            f"seed={self.seed}",
        ]
        return f"<ShortHashSeed [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .base import Opaque

__all__ = ["HmacSha256Mac"]


class HmacSha256Mac:
    """
    XDR Source Code::

        struct HmacSha256Mac
        {
            opaque mac[32];
        };
    """

    def __init__(
        self,
        mac: bytes,
    ) -> None:
        self.mac = mac

    def pack(self, packer: Packer) -> None:
        Opaque(self.mac, 32, True).pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> HmacSha256Mac:
        mac = Opaque.unpack(unpacker, 32, True)
        return cls(
            mac=mac,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> HmacSha256Mac:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> HmacSha256Mac:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash((self.mac,))

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.mac == other.mac

    def __repr__(self):
        out = [
            f"mac={self.mac}",
        ]
        return f"<HmacSha256Mac [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .liquidity_pool_withdraw_result_code import LiquidityPoolWithdrawResultCode

__all__ = ["LiquidityPoolWithdrawResult"]


class LiquidityPoolWithdrawResult:
    """
    XDR Source Code::

        union LiquidityPoolWithdrawResult switch (LiquidityPoolWithdrawResultCode code)
        {
        case LIQUIDITY_POOL_WITHDRAW_SUCCESS:
            void;
        case LIQUIDITY_POOL_WITHDRAW_MALFORMED:
        case LIQUIDITY_POOL_WITHDRAW_NO_TRUST:
        case LIQUIDITY_POOL_WITHDRAW_UNDERFUNDED:
        case LIQUIDITY_POOL_WITHDRAW_LINE_FULL:
        case LIQUIDITY_POOL_WITHDRAW_UNDER_MINIMUM:
            void;
        };
    """

    def __init__(
        self,
        code: LiquidityPoolWithdrawResultCode,
    ) -> None:
        self.code = code

    def pack(self, packer: Packer) -> None:
        self.code.pack(packer)
        if self.code == LiquidityPoolWithdrawResultCode.LIQUIDITY_POOL_WITHDRAW_SUCCESS:
            return
        if (
            self.code
            == LiquidityPoolWithdrawResultCode.LIQUIDITY_POOL_WITHDRAW_MALFORMED
        ):
            return
        if (
            self.code
            == LiquidityPoolWithdrawResultCode.LIQUIDITY_POOL_WITHDRAW_NO_TRUST
        ):
            return
        if (
            self.code
            == LiquidityPoolWithdrawResultCode.LIQUIDITY_POOL_WITHDRAW_UNDERFUNDED
        ):
            return
        if (
            self.code
            == LiquidityPoolWithdrawResultCode.LIQUIDITY_POOL_WITHDRAW_LINE_FULL
        ):
            return
        if (
            self.code
            == LiquidityPoolWithdrawResultCode.LIQUIDITY_POOL_WITHDRAW_UNDER_MINIMUM
        ):
            return

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> LiquidityPoolWithdrawResult:
        code = LiquidityPoolWithdrawResultCode.unpack(unpacker)
        if code == LiquidityPoolWithdrawResultCode.LIQUIDITY_POOL_WITHDRAW_SUCCESS:
            return cls(code=code)
        if code == LiquidityPoolWithdrawResultCode.LIQUIDITY_POOL_WITHDRAW_MALFORMED:
            return cls(code=code)
        if code == LiquidityPoolWithdrawResultCode.LIQUIDITY_POOL_WITHDRAW_NO_TRUST:
            return cls(code=code)
        if code == LiquidityPoolWithdrawResultCode.LIQUIDITY_POOL_WITHDRAW_UNDERFUNDED:
            return cls(code=code)
        if code == LiquidityPoolWithdrawResultCode.LIQUIDITY_POOL_WITHDRAW_LINE_FULL:
            return cls(code=code)
        if (
            code
            == LiquidityPoolWithdrawResultCode.LIQUIDITY_POOL_WITHDRAW_UNDER_MINIMUM
        ):
            return cls(code=code)
        return cls(code=code)

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> LiquidityPoolWithdrawResult:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> LiquidityPoolWithdrawResult:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash((self.code,))

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.code == other.code

    def __repr__(self):
        out = []
        out.append(f"code={self.code}")
        return f"<LiquidityPoolWithdrawResult [{', '.join(out)}]>"



from typing import Sequence

from ...asset import Asset
from ...call_builder.base.base_call_builder import BaseCallBuilder
from ...utils import convert_assets_to_horizon_param

__all__ = ["BaseLiquidityPoolsBuilder"]


class BaseLiquidityPoolsBuilder(BaseCallBuilder):
    """Creates a new :class:`LiquidityPoolsBuilder` pointed to server defined by horizon_url.

    See `List Liquidity Pools <https://developers.stellar.org/api/resources/liquiditypools/list/>`__ for more information.

    :param horizon_url: Horizon server URL.
    """

    def __init__(self, horizon_url: str) -> None:
        super().__init__(horizon_url)
        self.endpoint: str = "liquidity_pools"

    def liquidity_pool(self, liquidity_pool_id: str):
        """Provides information on a liquidity pool.

        See `Retrieve a Liquidity Pool <https://developers.stellar.org/api/resources/liquiditypools/single/>`__ for more information.

        :param liquidity_pool_id: The ID of the liquidity pool in hex string.
        :return: current LiquidityPoolsBuilder instance
        """
        self.endpoint = f"liquidity_pools/{liquidity_pool_id}"
        return self

    def for_reserves(self, reserves: Sequence[Asset]):
        """Get pools by reserves.

        Horizon will provide an endpoint to find all liquidity pools
        which contain a given set of reserve assets.

        See `List Liquidity Pools <https://developers.stellar.org/api/resources/liquiditypools/list/>`__ for more information.

        :return: current LiquidityPoolsBuilder instance
        """
        reserves_param = convert_assets_to_horizon_param(reserves)
        self._add_query_param("reserves", reserves_param)
        return self

    def for_account(self, account_id: str):
        """Filter pools for a specific account

        See `List Liquidity Pools <https://developers.stellar.org/api/resources/liquiditypools/list/>`__ for more information.

        :param account_id: account id
        :return: current LiquidityPoolsBuilder instance
        """
        self._add_query_param("account", account_id)
        return self



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .account_id import AccountID
from .data_entry_ext import DataEntryExt
from .data_value import DataValue
from .string64 import String64

__all__ = ["DataEntry"]


class DataEntry:
    """
    XDR Source Code::

        struct DataEntry
        {
            AccountID accountID; // account this data belongs to
            string64 dataName;
            DataValue dataValue;

            // reserved for future use
            union switch (int v)
            {
            case 0:
                void;
            }
            ext;
        };
    """

    def __init__(
        self,
        account_id: AccountID,
        data_name: String64,
        data_value: DataValue,
        ext: DataEntryExt,
    ) -> None:
        self.account_id = account_id
        self.data_name = data_name
        self.data_value = data_value
        self.ext = ext

    def pack(self, packer: Packer) -> None:
        self.account_id.pack(packer)
        self.data_name.pack(packer)
        self.data_value.pack(packer)
        self.ext.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> DataEntry:
        account_id = AccountID.unpack(unpacker)
        data_name = String64.unpack(unpacker)
        data_value = DataValue.unpack(unpacker)
        ext = DataEntryExt.unpack(unpacker)
        return cls(
            account_id=account_id,
            data_name=data_name,
            data_value=data_value,
            ext=ext,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> DataEntry:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> DataEntry:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.account_id,
                self.data_name,
                self.data_value,
                self.ext,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.account_id == other.account_id
            and self.data_name == other.data_name
            and self.data_value == other.data_value
            and self.ext == other.ext
        )

    def __repr__(self):
        out = [
            f"account_id={self.account_id}",
            f"data_name={self.data_name}",
            f"data_value={self.data_value}",
            f"ext={self.ext}",
        ]
        return f"<DataEntry [{', '.join(out)}]>"



from decimal import Decimal
from typing import Any, Coroutine, Dict, Generator, List, Tuple, Union

from .account import Account
from .asset import Asset
from .base_transaction_envelope import BaseTransactionEnvelope
from .call_builder.base import (
    BaseAccountsCallBuilder,
    BaseAssetsCallBuilder,
    BaseClaimableBalancesCallBuilder,
    BaseDataCallBuilder,
    BaseEffectsCallBuilder,
    BaseFeeStatsCallBuilder,
    BaseLedgersCallBuilder,
    BaseLiquidityPoolsBuilder,
    BaseOffersCallBuilder,
    BaseOperationsCallBuilder,
    BaseOrderbookCallBuilder,
    BasePaymentsCallBuilder,
    BaseRootCallBuilder,
    BaseStrictReceivePathsCallBuilder,
    BaseStrictSendPathsCallBuilder,
    BaseTradeAggregationsCallBuilder,
    BaseTradesCallBuilder,
    BaseTransactionsCallBuilder,
)
from .fee_bump_transaction import FeeBumpTransaction
from .fee_bump_transaction_envelope import FeeBumpTransactionEnvelope
from .helpers import parse_transaction_envelope_from_xdr
from .keypair import Keypair
from .muxed_account import MuxedAccount
from .operation import (
    AccountMerge,
    PathPaymentStrictReceive,
    PathPaymentStrictSend,
    Payment,
)
from .sep.exceptions import AccountRequiresMemoError
from .transaction import Transaction
from .transaction_envelope import TransactionEnvelope

__all__ = ["BaseServer"]


class BaseServer:
    def submit_transaction(
        self,
        transaction_envelope: Union[
            TransactionEnvelope, FeeBumpTransactionEnvelope, str
        ],
        skip_memo_required_check: bool = False,
    ) -> Union[Dict[str, Any], Coroutine[Any, Any, Dict[str, Any]]]:
        """Submits a transaction to the network.

        :param transaction_envelope: :class:`stellar_sdk.transaction_envelope.TransactionEnvelope` object
            or base64 encoded xdr
        :param skip_memo_required_check: Allow skipping memo
        :return: the response from horizon
        :raises:
            :exc:`ConnectionError <stellar_sdk.exceptions.ConnectionError>`
            :exc:`NotFoundError <stellar_sdk.exceptions.NotFoundError>`
            :exc:`BadRequestError <stellar_sdk.exceptions.BadRequestError>`
            :exc:`BadResponseError <stellar_sdk.exceptions.BadResponseError>`
            :exc:`UnknownRequestError <stellar_sdk.exceptions.UnknownRequestError>`
            :exc:`AccountRequiresMemoError <stellar_sdk.sep.exceptions.AccountRequiresMemoError>`
        """
        raise NotImplementedError

    def _get_xdr_and_transaction_from_transaction_envelope(
        self,
        transaction_envelope: Union[
            TransactionEnvelope, FeeBumpTransactionEnvelope, str
        ],
    ) -> Tuple[str, Union[Transaction, FeeBumpTransaction]]:
        if isinstance(transaction_envelope, BaseTransactionEnvelope):
            xdr = transaction_envelope.to_xdr()
            tx = transaction_envelope.transaction
        else:
            xdr = transaction_envelope
            tx = parse_transaction_envelope_from_xdr(
                transaction_envelope, ""
            ).transaction
        return xdr, tx

    def root(self) -> BaseRootCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.RootCallBuilder` object configured
            by a current Horizon server configuration.
        """
        raise NotImplementedError

    def accounts(self) -> BaseAccountsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.AccountsCallBuilder` object configured
            by a current Horizon server configuration.
        """
        raise NotImplementedError

    def assets(self) -> BaseAssetsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.AssetsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def claimable_balances(self) -> BaseClaimableBalancesCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.ClaimableBalancesCallBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def data(self, account_id: str, data_name: str) -> BaseDataCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.DataCallBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def effects(self) -> BaseEffectsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.EffectsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def fee_stats(self) -> BaseFeeStatsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.FeeStatsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def ledgers(self) -> BaseLedgersCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.LedgersCallBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def liquidity_pools(self) -> BaseLiquidityPoolsBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.LiquidityPoolsBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def offers(self) -> BaseOffersCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.OffersCallBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def operations(self) -> BaseOperationsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.OperationsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def orderbook(self, selling: Asset, buying: Asset) -> BaseOrderbookCallBuilder:
        """
        :param selling: Asset being sold
        :param buying: Asset being bought
        :return: New :class:`stellar_sdk.call_builder.OrderbookCallBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def strict_receive_paths(
        self,
        source: Union[str, List[Asset]],
        destination_asset: Asset,
        destination_amount: Union[str, Decimal],
    ) -> BaseStrictReceivePathsCallBuilder:
        """
        :param source: The sender's account ID or a list of Assets. Any returned path must use a source that the sender can hold.
        :param destination_asset: The destination asset.
        :param destination_amount: The amount, denominated in the destination asset, that any returned path should be able to satisfy.
        :return: New :class:`stellar_sdk.call_builder.StrictReceivePathsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def strict_send_paths(
        self,
        source_asset: Asset,
        source_amount: Union[str, Decimal],
        destination: Union[str, List[Asset]],
    ) -> BaseStrictSendPathsCallBuilder:
        """
        :param source_asset: The asset to be sent.
        :param source_amount: The amount, denominated in the source asset, that any returned path should be able to satisfy.
        :param destination: The destination account or the destination assets.
        :return: New :class:`stellar_sdk.call_builder.StrictReceivePathsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def payments(self) -> BasePaymentsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.PaymentsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def trade_aggregations(
        self,
        base: Asset,
        counter: Asset,
        resolution: int,
        start_time: int = None,
        end_time: int = None,
        offset: int = None,
    ) -> BaseTradeAggregationsCallBuilder:
        """
        :param base: base asset
        :param counter: counter asset
        :param resolution: segment duration as millis since epoch. *Supported values
            are 1 minute (60000), 5 minutes (300000), 15 minutes (900000),
            1 hour (3600000), 1 day (86400000) and 1 week (604800000).*
        :param start_time: lower time boundary represented as millis since epoch
        :param end_time: upper time boundary represented as millis since epoch
        :param offset: segments can be offset using this parameter.
            Expressed in milliseconds. *Can only be used if the resolution is greater than 1 hour.
            Value must be in whole hours, less than the provided resolution, and less than 24 hours.*
        :return: New :class:`stellar_sdk.call_builder.TradeAggregationsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def trades(self) -> BaseTradesCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.TradesCallBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def transactions(self) -> BaseTransactionsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.TransactionsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        raise NotImplementedError

    def load_account(
        self, account_id: Union[MuxedAccount, Keypair, str]
    ) -> Union[Account, Coroutine[Any, Any, Account]]:
        """Fetches an account's most current base state (like sequence) in the ledger and then creates
        and returns an :class:`stellar_sdk.account.Account` object.

        If you want to get complete account information, please
        use :func:`stellar_sdk.server.Server.accounts`.

        :param account_id: The account to load.
        :return: an :class:`stellar_sdk.account.Account` object.
        :raises:
            :exc:`ConnectionError <stellar_sdk.exceptions.ConnectionError>`
            :exc:`NotFoundError <stellar_sdk.exceptions.NotFoundError>`
            :exc:`BadRequestError <stellar_sdk.exceptions.BadRequestError>`
            :exc:`BadResponseError <stellar_sdk.exceptions.BadResponseError>`
            :exc:`UnknownRequestError <stellar_sdk.exceptions.UnknownRequestError>`
        """
        raise NotImplementedError

    def _check_destination_memo(
        self, account_resp: dict, index: int, destination: str
    ) -> None:
        memo_required_config_key = "config.memo_required"
        memo_required_config_value = "MQ=="
        data = account_resp["data"]
        if data.get(memo_required_config_key) == memo_required_config_value:
            raise AccountRequiresMemoError(
                "Destination account requires a memo in the transaction.",
                destination,
                index,
            )

    def _get_check_memo_required_destinations(
        self, transaction: Transaction
    ) -> Generator[Tuple[int, str], Any, Any]:
        destinations = set()
        for index, operation in enumerate(transaction.operations):
            if isinstance(
                operation,
                (
                    Payment,
                    AccountMerge,
                    PathPaymentStrictSend,
                    PathPaymentStrictReceive,
                ),
            ):
                destination: str = operation.destination.account_id
            else:
                continue
            if destination in destinations:
                continue
            destinations.add(destination)
            yield index, destination

    def fetch_base_fee(self) -> Union[int, Coroutine[Any, Any, int]]:
        """Fetch the base fee. Since this hits the server, if the server call fails,
        you might get an error. You should be prepared to use a default value if that happens.

        :return: the base fee
        :raises:
            :exc:`ConnectionError <stellar_sdk.exceptions.ConnectionError>`
            :exc:`NotFoundError <stellar_sdk.exceptions.NotFoundError>`
            :exc:`BadRequestError <stellar_sdk.exceptions.BadRequestError>`
            :exc:`BadResponseError <stellar_sdk.exceptions.BadResponseError>`
            :exc:`UnknownRequestError <stellar_sdk.exceptions.UnknownRequestError>`
        """
        raise NotImplementedError

    def _handle_base_fee(self, latest_ledger: dict) -> int:
        base_fee = 100
        if (
            latest_ledger["_embedded"]
            and latest_ledger["_embedded"]["records"]
            and latest_ledger["_embedded"]["records"][0]
        ):
            base_fee = int(
                latest_ledger["_embedded"]["records"][0]["base_fee_in_stroops"]
            )
        return base_fee

    def close(self) -> Union[None, Coroutine[Any, Any, None]]:
        """Close underlying connector.

        Release all acquired resources.
        """
        raise NotImplementedError

    async def __aenter__(self):
        raise NotImplementedError

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        raise NotImplementedError

    def __enter__(self):
        raise NotImplementedError

    def __exit__(self, exc_type, exc_val, exc_tb):
        raise NotImplementedError



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .extension_point import ExtensionPoint
from .int64 import Int64

__all__ = ["SorobanTransactionMetaExtV1"]


class SorobanTransactionMetaExtV1:
    """
    XDR Source Code::

        struct SorobanTransactionMetaExtV1
        {
            ExtensionPoint ext;

            // The following are the components of the overall Soroban resource fee
            // charged for the transaction.
            // The following relation holds:
            // `resourceFeeCharged = totalNonRefundableResourceFeeCharged + totalRefundableResourceFeeCharged`
            // where `resourceFeeCharged` is the overall fee charged for the
            // transaction. Also, `resourceFeeCharged` <= `sorobanData.resourceFee`
            // i.e.we never charge more than the declared resource fee.
            // The inclusion fee for charged the Soroban transaction can be found using
            // the following equation:
            // `result.feeCharged = resourceFeeCharged + inclusionFeeCharged`.

            // Total amount (in stroops) that has been charged for non-refundable
            // Soroban resources.
            // Non-refundable resources are charged based on the usage declared in
            // the transaction envelope (such as `instructions`, `readBytes` etc.) and
            // is charged regardless of the success of the transaction.
            int64 totalNonRefundableResourceFeeCharged;
            // Total amount (in stroops) that has been charged for refundable
            // Soroban resource fees.
            // Currently this comprises the rent fee (`rentFeeCharged`) and the
            // fee for the events and return value.
            // Refundable resources are charged based on the actual resources usage.
            // Since currently refundable resources are only used for the successful
            // transactions, this will be `0` for failed transactions.
            int64 totalRefundableResourceFeeCharged;
            // Amount (in stroops) that has been charged for rent.
            // This is a part of `totalNonRefundableResourceFeeCharged`.
            int64 rentFeeCharged;
        };
    """

    def __init__(
        self,
        ext: ExtensionPoint,
        total_non_refundable_resource_fee_charged: Int64,
        total_refundable_resource_fee_charged: Int64,
        rent_fee_charged: Int64,
    ) -> None:
        self.ext = ext
        self.total_non_refundable_resource_fee_charged = (
            total_non_refundable_resource_fee_charged
        )
        self.total_refundable_resource_fee_charged = (
            total_refundable_resource_fee_charged
        )
        self.rent_fee_charged = rent_fee_charged

    def pack(self, packer: Packer) -> None:
        self.ext.pack(packer)
        self.total_non_refundable_resource_fee_charged.pack(packer)
        self.total_refundable_resource_fee_charged.pack(packer)
        self.rent_fee_charged.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> SorobanTransactionMetaExtV1:
        ext = ExtensionPoint.unpack(unpacker)
        total_non_refundable_resource_fee_charged = Int64.unpack(unpacker)
        total_refundable_resource_fee_charged = Int64.unpack(unpacker)
        rent_fee_charged = Int64.unpack(unpacker)
        return cls(
            ext=ext,
            total_non_refundable_resource_fee_charged=total_non_refundable_resource_fee_charged,
            total_refundable_resource_fee_charged=total_refundable_resource_fee_charged,
            rent_fee_charged=rent_fee_charged,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> SorobanTransactionMetaExtV1:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> SorobanTransactionMetaExtV1:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.ext,
                self.total_non_refundable_resource_fee_charged,
                self.total_refundable_resource_fee_charged,
                self.rent_fee_charged,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.ext == other.ext
            and self.total_non_refundable_resource_fee_charged
            == other.total_non_refundable_resource_fee_charged
            and self.total_refundable_resource_fee_charged
            == other.total_refundable_resource_fee_charged
            and self.rent_fee_charged == other.rent_fee_charged
        )

    def __repr__(self):
        out = [
            f"ext={self.ext}",
            f"total_non_refundable_resource_fee_charged={self.total_non_refundable_resource_fee_charged}",
            f"total_refundable_resource_fee_charged={self.total_refundable_resource_fee_charged}",
            f"rent_fee_charged={self.rent_fee_charged}",
        ]
        return f"<SorobanTransactionMetaExtV1 [{', '.join(out)}]>"
